@@ -1,7 +1,7 @@
 // -*-c++-*-
 
 /*!
-  \file body_dribble.h
+  \file body_dribble_test.h
   \brief advanced dribble action. player agent can avoid opponent.
 */
 
@@ -29,13 +29,14 @@
 
 /////////////////////////////////////////////////////////////////////
 
-#ifndef RCSC_ACTION_BODY_DRIBBLE_H
-#define RCSC_ACTION_BODY_DRIBBLE_H
+#ifndef RCSC_ACTION_BODY_DRIBBLE_2007_H
+#define RCSC_ACTION_BODY_DRIBBLE_2007_H
 
 #include <rcsc/player/soccer_action.h>
 #include <rcsc/geom/vector_2d.h>
 
-#include <functional>
+#include <vector>
+#include <list>
 
 namespace rcsc {
 
@@ -51,38 +52,21 @@ public:
 
     struct KeepDribbleInfo {
         Vector2D first_ball_vel_;
+        double ball_forward_travel_;
         int dash_count_;
         double min_opp_dist_;
 
         KeepDribbleInfo( const Vector2D & first_ball_vel,
+                         const double & ball_forward_travel,
                          const int dash_count,
                          const double & min_opp_dist )
             : first_ball_vel_( first_ball_vel )
+            , ball_forward_travel_( ball_forward_travel )
             , dash_count_( dash_count )
             , min_opp_dist_( min_opp_dist )
           { }
     };
 
-    struct KeepDribbleCmp
-        : public std::binary_function< KeepDribbleInfo,
-                                       KeepDribbleInfo,
-                                       bool > {
-
-        result_type operator()( const first_argument_type & lhs,
-                                const second_argument_type & rhs ) const
-          {
-              if ( lhs.dash_count_ > rhs.dash_count_ )
-              {
-                  return true;
-              }
-              if ( lhs.dash_count_ == rhs.dash_count_ )
-              {
-                  return ( lhs.min_opp_dist_ < rhs.min_opp_dist_ );
-              }
-              return false;
-          }
-    };
-
 private:
     //! target point to be reached
     const Vector2D M_target_point;
@@ -122,7 +106,7 @@ public:
       \param agent pointer to the agent itself
       \return true if action is performed
     */
-    bool execute( PlayerAgent* agent );
+    bool execute( PlayerAgent * agent );
 
 private:
 
@@ -140,7 +124,14 @@ private:
                    const Vector2D & target_point,
                    const double & dash_power,
                    const int dash_count,
-                   const bool dodge );
+                   const bool dodge,
+                   const bool dodge_mode = false );
+
+    bool doTurn( PlayerAgent * agent,
+                 const Vector2D & target_point,
+                 const double & dash_power,
+                 const int dash_count,
+                 const bool dodge );
 
     /*!
       \brief if possible perform turn action only
@@ -205,7 +196,15 @@ private:
       but ball should be always keep.
      */
     bool doKickDashesWithBall( PlayerAgent * agent,
-                               const double & dash_power );
+                               const Vector2D & target_point,
+                               const double & dash_power,
+                               const int dash_count,
+                               const bool dodge_mode );
+
+    void createSelfCache( PlayerAgent * agent,
+                          const double & dash_power,
+                          const int dash_count,
+                          std::vector< Vector2D > & self_cache );
 
     bool existKickableOpponent( const WorldModel & wm,
                                 const Vector2D & ball_pos,
