@@ -44,7 +44,7 @@
 #include <rcsc/player/debug_client.h>
 
 #include <rcsc/geom/line_2d.h>
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/server_param.h>
 
 namespace rcsc {
 
@@ -84,13 +84,11 @@ Body_HoldBall::execute( PlayerAgent * agent )
 
     if ( avoidOpponent( agent, front_keep_dist ) )
     {
-        agent->debugClient().addMessage( "Hold:exeTurn" );
         return true;
     }
 
     if ( turnToPoint( agent ) )
     {
-        agent->debugClient().addMessage( "Hold:exeFront" );
         return true;
     }
 
@@ -205,9 +203,9 @@ Body_HoldBall::avoidOpponent( PlayerAgent * agent,
         {
             keep_dist = my_kickable + 0.1;
             dlog.addText( Logger::ACTION,
-                          "%s%d:: Body_HoldBall. slightly kick out. dist= %.2f"
+                          "%s%d: Hold. slightly kick out. keep_dist=%.2f keep_angle=%.0f"
                           ,__FILE__, __LINE__,
-                          keep_dist );
+                          keep_dist, keep_angle.degree() );
         }
         else
         {
@@ -216,17 +214,17 @@ Body_HoldBall::avoidOpponent( PlayerAgent * agent,
                 + wm.self().playerType().kickableMargin()
                 - 0.06;
             dlog.addText( Logger::ACTION,
-                          "%s:%d: Body_HoldBall. at kickable edge. dist= %.2f"
+                          "%s:%d: Hold. at kickable edge. keep_dist=%.2f keep_angle=%.0f"
                           ,__FILE__, __LINE__,
-                          keep_dist );
+                          keep_dist, keep_angle.degree() );
         }
     }
     else
     {
-        Vector2D keep_pos
+        Vector2D tmp_pos
             = Vector2D::polar2vector( keep_dist, keep_angle );
-        keep_pos += wm.self().vel();
-        double ball_travel = ( wm.ball().rpos() - keep_pos ).r();
+        tmp_pos += wm.self().vel();
+        double ball_travel = ( wm.ball().rpos() - tmp_pos ).r();
         keep_dist
             = wm.self().playerType().playerSize()
             + wm.self().playerType().kickableMargin()
@@ -234,18 +232,22 @@ Body_HoldBall::avoidOpponent( PlayerAgent * agent,
             - ball_travel * ServerParam::i().ballRand()
             - 0.15;
         dlog.addText( Logger::ACTION,
-                      "%s: Body_HoldBall. keep_dist= %.2f. ball_travel= %.2f"
+                      "%s: HoldBall. set buffer. keep_dist=%.2f. keep_angle=%.0f"
+                      " ball_travel= %.2f"
                       ,__FILE__,
-                      keep_dist, ball_travel );
+                      keep_dist, keep_angle.degree(), ball_travel );
     }
 
     ////////////////////////////////////////////////////////////////////
     // execute action
-    dlog.addText( Logger::ACTION,
-                  "%s:%d: HOLD. keep dist= %.2f.  angle= %.1f"
-                  ,__FILE__, __LINE__,
-                  keep_dist, keep_angle.degree() );
+    Vector2D keep_pos
+        = wm.self().pos()
+        + wm.self().vel()
+        + Vector2D::polar2vector( keep_dist, keep_angle );
+
     agent->debugClient().addMessage( "Hold:%.1f", keep_angle.degree() );
+    agent->debugClient().addCircle( keep_pos, 0.05 );
+
     return Body_KickToRelative( keep_dist,
                                 keep_angle - wm.self().body(),
                                 false // not stop the ball
@@ -272,9 +274,20 @@ Body_HoldBall::avoidOpponentLine( PlayerAgent * agent,
         Vector2D my_next = wm.self().pos() + wm.self().vel();
         Vector2D proj_pos = opp_line.projection( my_next );
         AngleDeg keep_angle = ( my_next - proj_pos ).th();
+        double keep_dist = wm.self().playerType().kickableArea() - 0.2;
         Vector2D keep_pos = my_next
-            + Vector2D::polar2vector( wm.self().playerType().kickableArea() - 0.25,
+            + Vector2D::polar2vector( keep_dist,
                                       keep_angle );
+        double ball_travel = wm.ball().pos().dist( keep_pos );
+        keep_dist
+            = wm.self().playerType().kickableArea()
+            - wm.self().vel().r() * ServerParam::i().playerRand()
+            - ball_travel * ServerParam::i().ballRand()
+            - 0.15;
+        keep_pos = my_next
+            + Vector2D::polar2vector( keep_dist,
+                                      keep_angle );
+
         dlog.addText( Logger::ACTION,
                       "%s:%d: avoid opponent line. keep_pos=(%.2f %.2f) angle=%.1f"
                       ,__FILE__, __LINE__,
@@ -284,12 +297,12 @@ Body_HoldBall::avoidOpponentLine( PlayerAgent * agent,
         {
             agent->debugClient().addMessage( "HoldAvoidOppBody%.0f",
                                              keep_angle.degree() );
+            agent->debugClient().addCircle( keep_pos, 0.05 );
             dlog.addText( Logger::ACTION,
                           "%s:%d: done avoid opponent line. "
                           ,__FILE__, __LINE__ );
-            agent->debugClient().addMessage( "AvoidHold:%.1f",
-                                             keep_angle.degree() );
-            Body_KickToRelative( wm.self().playerType().kickableArea()  - 0.2,
+
+            Body_KickToRelative( keep_dist,
                                  keep_angle - wm.self().body(),
                                  false // not need to stop the ball
                                  ).execute( agent );
