@@ -43,7 +43,7 @@
 
 #include <rcsc/player/logger.h>
 #include <rcsc/player/player_agent.h>
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/server_param.h>
 #include <rcsc/soccer_math.h>
 
 namespace rcsc {
@@ -70,10 +70,6 @@ Body_KickMultiStep::execute( PlayerAgent * agent )
         return false;
     }
 
-    Vector2D required_acc;
-    double kick_pow;
-    AngleDeg kick_dir;
-
     //-------------------------------------------
     Vector2D target_rpos = M_target_point - agent->world().self().pos();
     M_first_speed = std::min( M_first_speed, ServerParam::i().ballSpeedMax() );
@@ -84,6 +80,8 @@ Body_KickMultiStep::execute( PlayerAgent * agent )
                   "%s:%d: simulate one kick"
                   ,__FILE__, __LINE__ );
     if ( Body_KickTwoStep::simulate_one_kick( &achieved_vel,
+                                              NULL,
+                                              NULL,
                                               target_rpos,
                                               M_first_speed,
                                               Vector2D( 0.0, 0.0 ), // my current pos
@@ -94,25 +92,27 @@ Body_KickMultiStep::execute( PlayerAgent * agent )
                                               agent,
                                               false ) ) // not enforce
     {
-        required_acc = achieved_vel - agent->world().ball().vel();
-        kick_pow = required_acc.r() / agent->world().self().kickRate();
-        kick_dir = required_acc.th() - agent->world().self().body();
+        Vector2D accel = achieved_vel - agent->world().ball().vel();
+        double kick_power = accel.r() / agent->world().self().kickRate();
+        AngleDeg kick_dir = accel.th() - agent->world().self().body();
 
         dlog.addText( Logger::KICK,
                       "%s:%d: only one kick"
-                      "   result=(%.3f, %.3f)[r=%.3f]  acc=(%.3f, %.3f)  power=%.1f dir=%.1f"
+                      "   result=(%.3f, %.3f)[r=%.3f]  accel=(%.3f, %.3f)  power=%.1f dir=%.1f"
                       ,__FILE__, __LINE__,
                       achieved_vel.x, achieved_vel.y, achieved_vel.r(),
-                      required_acc.x, required_acc.y,
-                      kick_pow, kick_dir.degree() );
-        return agent->doKick( kick_pow, kick_dir );
+                      accel.x, accel.y,
+                      kick_power, kick_dir.degree() );
+        return agent->doKick( kick_power, kick_dir );
     }
 
 
-    Vector2D next_vel;
     dlog.addText( Logger::KICK,
                   "%s:%d: simulate two kick"
                   ,__FILE__, __LINE__ );
+
+    Vector2D next_vel;
+
     if ( Body_KickTwoStep::simulate_two_kick( &achieved_vel,
                                               &next_vel,
                                               target_rpos,
@@ -125,9 +125,9 @@ Body_KickMultiStep::execute( PlayerAgent * agent )
                                               agent,
                                               false ) ) // not enforced
     {
-        required_acc = next_vel - agent->world().ball().vel();
-        kick_pow = required_acc.r() / agent->world().self().kickRate();
-        kick_dir = required_acc.th() - agent->world().self().body();
+        Vector2D accel = next_vel - agent->world().ball().vel();
+        double kick_power = accel.r() / agent->world().self().kickRate();
+        AngleDeg kick_dir = accel.th() - agent->world().self().body();
 
         dlog.addText( Logger::KICK,
                       "%s:%d: two kick."
@@ -136,14 +136,17 @@ Body_KickMultiStep::execute( PlayerAgent * agent )
                       ,__FILE__, __LINE__,
                       achieved_vel.x, achieved_vel.y, achieved_vel.r(),
                       next_vel.x, next_vel.y, next_vel.r(),
-                      required_acc.x, required_acc.y,
-                      kick_pow, kick_dir.degree() );
-        return agent->doKick( kick_pow, kick_dir );
+                      accel.x, accel.y,
+                      kick_power, kick_dir.degree() );
+        agent->debugClient().addCircle( agent->world().ball().pos() + next_vel,
+                                        0.05 );
+        return agent->doKick( kick_power, kick_dir );
     }
 
     dlog.addText( Logger::KICK,
                   "%s:%d: simulate three kick"
                   ,__FILE__, __LINE__ );
+
     if ( simulate_three_kick( &achieved_vel,
                               &next_vel,
                               target_rpos,
@@ -156,9 +159,9 @@ Body_KickMultiStep::execute( PlayerAgent * agent )
                               agent,
                               false ) )
     {
-        required_acc = next_vel - agent->world().ball().vel();
-        kick_pow = required_acc.r() / agent->world().self().kickRate();
-        kick_dir = required_acc.th() - agent->world().self().body();
+        Vector2D accel = next_vel - agent->world().ball().vel();
+        double kick_power = accel.r() / agent->world().self().kickRate();
+        AngleDeg kick_dir = accel.th() - agent->world().self().body();
 
         dlog.addText( Logger::KICK,
                       "%s:%d: three kick."
@@ -167,9 +170,11 @@ Body_KickMultiStep::execute( PlayerAgent * agent )
                       ,__FILE__, __LINE__,
                       achieved_vel.x, achieved_vel.y, achieved_vel.r(),
                       next_vel.x, next_vel.y, next_vel.r(),
-                      required_acc.x, required_acc.y,
-                      kick_pow, kick_dir.degree() );
-        return agent->doKick( kick_pow, kick_dir );
+                      accel.x, accel.y,
+                      kick_power, kick_dir.degree() );
+        agent->debugClient().addCircle( agent->world().ball().pos() + next_vel,
+                                        0.05 );
+        return agent->doKick( kick_power, kick_dir );
     }
 
 
@@ -254,7 +259,7 @@ Body_KickMultiStep::simulate_three_kick( Vector2D * achieved_vel,
                 + Vector2D::polar2vector(subtarget_dist,
                                          first_sub_target_angle + add_dir);
             Vector2D require_vel = sub - ball_rpos;
-            if ( ! Body_KickTwoStep::is_opp_kickable( agent, sub ) // check opponents
+            if ( ! Body_KickTwoStep::is_opp_kickable( agent, sub, NULL )
                  && (require_vel - ball_vel).r() < enable_acc )
             {
                 subtargets.push_back
