@@ -118,7 +118,7 @@ public:
       \param world consr rerefence to the WorldModel
       \param target_point receive target point is stored to this
       \param first_speed ball first speed is stored to this
-      \param receiver receive number is stored to this
+      \param receiver receiver number
       \return true if pass route is found.
     */
     static
