@@ -35,15 +35,15 @@
 
 #include "intention_dribble.h"
 
-#include "basic_actions.h"
-#include "body_go_to_point.h"
-#include "neck_scan_field.h"
-#include "neck_turn_to_goalie_or_scan.h"
+#include <rcsc/action/basic_actions.h>
+#include <rcsc/action/body_go_to_point.h>
+#include <rcsc/action/neck_scan_field.h>
+#include <rcsc/action/neck_turn_to_goalie_or_scan.h>
 
 #include <rcsc/player/logger.h>
 #include <rcsc/player/player_agent.h>
 #include <rcsc/player/debug_client.h>
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/server_param.h>
 #include <rcsc/geom/line_2d.h>
 
 namespace rcsc {
@@ -110,8 +110,8 @@ IntentionDribble::execute( PlayerAgent * agent )
     const WorldModel & wm = agent->world();
 
     const Vector2D ball_next = wm.ball().pos() + wm.ball().vel();
-    if ( ball_next.absX() > ServerParam::i().pitchHalfLength() - 0.15
-         || ball_next.absY() > ServerParam::i().pitchHalfWidth() - 0.15 )
+    if ( ball_next.absX() > ServerParam::i().pitchHalfLength() - 0.5
+         || ball_next.absY() > ServerParam::i().pitchHalfWidth() - 0.5 )
     {
         dlog.addText( Logger::DRIBBLE,
                       "%s:%d: execute(). ball will be out of pitch. stop intention."
@@ -252,9 +252,9 @@ IntentionDribble::checkOpponent( const WorldModel & world )
                                 + ServerParam::i().defaultKickableArea() + 0.3 ) )
     {
         const Vector2D opp_next = nearest_opp->pos() + nearest_opp->vel();
-        // known opp angle
-        if ( ( nearest_opp->bodyCount() == 0
-               || nearest_opp->vel().r() > 0.2 ) )
+        // oppopnent angle is known
+        if ( nearest_opp->bodyCount() == 0
+             || nearest_opp->vel().r() > 0.2 )
         {
             Line2D opp_line( opp_next,
                              ( nearest_opp->bodyCount() == 0
@@ -267,22 +267,23 @@ IntentionDribble::checkOpponent( const WorldModel & world )
                               "%s:%d: existOpponent(). opp never reach."
                               ,__FILE__, __LINE__ );
             }
-            else
+            else if ( opp_next.dist( ball_next ) < 0.6 + 1.2 )
             {
-                if ( opp_next.dist( ball_next ) < 0.6 + 1.2 )
-                {
-                    dlog.addText( Logger::DRIBBLE,
-                                  "%s:%d: existOpponent(). but opp may reachable(1)."
-                                  ,__FILE__, __LINE__ );
-                    this->clear();
-                    return true;
-                }
+                dlog.addText( Logger::DRIBBLE,
+                              "%s:%d: existOpponent(). but opp may reachable(1)."
+                              ,__FILE__, __LINE__ );
+                this->clear();
+                return true;
             }
+
+            dlog.addText( Logger::DRIBBLE,
+                          "%s:%d: existOpponent(). opp angle is known. opp may not be reached."
+                          ,__FILE__, __LINE__ );
         }
-        // dont know opp angle
-        else if ( nearest_opp->vel().r() < 0.1 )
+        // opponent angle is not known
+        else
         {
-            if ( opp_next.dist( ball_next ) < 0.6 + 1.2 )
+            if ( opp_next.dist( ball_next ) < 1.2 + 1.2 ) //0.6 + 1.2 )
             {
                 dlog.addText( Logger::DRIBBLE,
                               "%s:%d: existOpponent(). but opp may reachable(2)."
@@ -356,7 +357,7 @@ IntentionDribble::doTurn( PlayerAgent * agent )
 bool
 IntentionDribble::doDash( PlayerAgent * agent )
 {
-    if ( M_dash_step <=0 )
+    if ( M_dash_step <= 0 )
     {
         return false;
     }
@@ -388,7 +389,8 @@ IntentionDribble::doDash( PlayerAgent * agent )
     Vector2D ball_next
         = wm.ball().rpos()
         + wm.ball().vel();
-    double ball_next_dist = my_next.dist( ball_next );
+    Vector2D ball_next_rel = ( ball_next - my_next ).rotatedVector( - accel_angle );
+    double ball_next_dist = ball_next_rel.r();
 
     if ( ball_next_dist
          < ( wm.self().playerType().playerSize()
@@ -396,11 +398,23 @@ IntentionDribble::doDash( PlayerAgent * agent )
          )
     {
         dlog.addText( Logger::DRIBBLE,
-                      "%s:%d: doDash. collision may occur."
-                      ,__FILE__, __LINE__ );
+                      "%s: doDash. collision may occur. ball_dist = %.2f"
+                      ,__FILE__,
+                      ball_next_dist );
         this->clear();
         return false;
     }
+
+    if ( ball_next_rel.absY() > wm.self().kickableArea() - 0.1 )
+    {
+        dlog.addText( Logger::DRIBBLE,
+                      "%s: doDash. next Y difference is over. y_diff = %.2f"
+                      ,__FILE__,
+                      ball_next_rel.absY() );
+        this->clear();
+        return false;
+    }
+
     // this dash is the last of queue
     // but at next cycle, ball is NOT kickable
     if ( M_dash_step <= 0 )
@@ -408,9 +422,9 @@ IntentionDribble::doDash( PlayerAgent * agent )
         if ( ball_next_dist > wm.self().kickableArea() - 0.15 )
         {
             dlog.addText( Logger::DRIBBLE,
-                          "%s:%d: doDash. last dash. but not kickable at next."
+                          "%s: doDash. last dash. but not kickable at next."
                           " ball_dist=%.3f"
-                          ,__FILE__, __LINE__,
+                          ,__FILE__,
                           ball_next_dist );
             this->clear();
             return false;
@@ -420,7 +434,8 @@ IntentionDribble::doDash( PlayerAgent * agent )
     if ( M_dash_step > 0 )
     {
         // remain dash step. but next dash cause over run.
-        if ( (accel_angle - wm.ball().angleFromSelf()).abs() > 100.0
+        AngleDeg ball_next_angle = ( ball_next - my_next ).th();
+        if ( ( accel_angle - ball_next_angle ).abs() > 90.0
              && ball_next_dist > wm.self().kickableArea() - 0.2 )
         {
             dlog.addText( Logger::DRIBBLE,
