@@ -41,7 +41,7 @@
 
 #include <rcsc/player/logger.h>
 #include <rcsc/player/player_agent.h>
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/server_param.h>
 
 namespace rcsc {
 
@@ -56,8 +56,31 @@ Neck_TurnToBallOrScan::execute( PlayerAgent * agent )
                   "%s:%d: Neck_TurnToBallOrScan"
                   ,__FILE__, __LINE__ );
 
+
+    const Vector2D ball_next = agent->effector().queuedNextBallPos();
+    const Vector2D my_next = agent->effector().queuedNextMyPos();
+    const AngleDeg my_next_body = agent->effector().queuedNextMyBody();
+    const double next_view_width = agent->effector().queuedNextViewWidth().width();
+
+    bool can_face = false;
+
+    if ( ( (ball_next - my_next).th() - my_next_body ).abs()
+         < ServerParam::i().maxNeckAngle() + next_view_width * 0.5 - 2.0 )
+    {
+        can_face = true;
+    }
+
+    if ( ! can_face )
+    {
+        dlog.addText( Logger::ACTION,
+                      "%s:%d: never face to ball"
+                      ,__FILE__, __LINE__ );
+        return Neck_ScanField().execute( agent );
+    }
+
     // ball is not seen at current
-    if ( agent->world().ball().posCount() != 0 )
+    if ( can_face
+         && agent->world().ball().posCount() != 0 )
     {
         dlog.addText( Logger::ACTION,
                       "%s:%d: simply face ball"
@@ -80,8 +103,6 @@ Neck_TurnToBallOrScan::execute( PlayerAgent * agent )
     // exist kickable player
     // they may accerelate the ball to max speed.
 
-    const Vector2D ball_next = agent->effector().queuedNextBallPos();
-
     if ( agent->world().ball().distFromSelf() < 20.0 )
     {
         dlog.addText( Logger::ACTION,
@@ -90,21 +111,9 @@ Neck_TurnToBallOrScan::execute( PlayerAgent * agent )
         return Neck_TurnToPoint( ball_next ).execute( agent );
     }
 
-    const Vector2D my_next = agent->effector().queuedNextMyPos();
-    const AngleDeg my_next_body = agent->effector().queuedNextMyBody();
+    // consider ball reachable dist after 2step
     const double narrow_width = ServerParam::i().visibleAngle() * 0.25; // half width
 
-    // ball point is NOT within neck range.
-    if ( ( (ball_next - my_next).th() - my_next_body ).abs()
-         > ServerParam::i().maxNeckAngle() + narrow_width )
-    {
-        dlog.addText( Logger::ACTION,
-                      "%s:%d: never face to ball"
-                      ,__FILE__, __LINE__ );
-        return Neck_ScanField().execute( agent );
-    }
-
-    // consider ball reachable dist after 2step
     double reachable_angle = std::fabs( AngleDeg::atan2_deg
                                         ( ServerParam::i().ballSpeedMax() * 2.0,
                                           ( ball_next - my_next ).r() ) );
