@@ -42,7 +42,7 @@
 #include <rcsc/player/logger.h>
 #include <rcsc/player/player_agent.h>
 #include <rcsc/player/debug_client.h>
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/server_param.h>
 
 namespace rcsc {
 
@@ -98,6 +98,11 @@ Neck_TurnToLowConfTeammate::execute( PlayerAgent * agent )
               it != end;
               ++it )
         {
+            if ( (*it)->isGhost() )
+            {
+                continue;
+            }
+
             if ( (*it)->distFromSelf() > max_dist )
             {
                 break;
@@ -133,8 +138,9 @@ Neck_TurnToLowConfTeammate::execute( PlayerAgent * agent )
     neck_moment = ServerParam::i().normalizeNeckMoment( neck_moment.degree() );
 
     dlog.addText( Logger::ACTION,
-                  "%s:%d: CheckTeammate candidate=(%.1f, %.1f). pos_count=%d"
+                  "%s:%d: look teammate %d (%.1f, %.1f). pos_count=%d"
                   ,__FILE__, __LINE__,
+                  candidate_unum,
                   candidate_point.x, candidate_point.y,
                   pos_count );
 
