@@ -40,9 +40,11 @@
 #include "global_visual_sensor.h"
 #include "global_world_model.h"
 
-#include <rcsc/param/server_param.h>
-#include <rcsc/param/player_param.h>
-#include <rcsc/param/player_type.h>
+#include <rcsc/common/basic_client.h>
+#include <rcsc/common/server_param.h>
+#include <rcsc/common/player_param.h>
+#include <rcsc/common/player_type.h>
+
 #include <rcsc/param/param_map.h>
 #include <rcsc/param/conf_file_parser.h>
 #include <rcsc/param/cmd_line_parser.h>
@@ -171,7 +173,7 @@ CoachAgentImpl::updateServerStatus()
 
 */
 CoachAgent::CoachAgent()
-    : BasicClient()
+    : SoccerAgent()
     , M_impl( new CoachAgentImpl )
 {
 
@@ -202,7 +204,7 @@ CoachAgent::visualSensor() const
 
 */
 bool
-CoachAgent::doInit( CmdLineParser & cmd_parser )
+CoachAgent::initImpl( CmdLineParser & cmd_parser )
 {
     bool help = false;
     std::string coach_config_file;
@@ -221,7 +223,7 @@ CoachAgent::doInit( CmdLineParser & cmd_parser )
     {
         system_param_map.printHelp( std::cout );
         coach_param_map.printHelp( std::cout );
-        setServerAlive( false );
+        M_client->setServerAlive( false );
         return false;
     }
 
@@ -248,79 +250,36 @@ CoachAgent::doInit( CmdLineParser & cmd_parser )
 /*!
 
 */
-void
-CoachAgent::doConnect()
+bool
+CoachAgent::handleStart()
 {
+    if ( ! M_client )
+    {
+        return false;
+    }
+
     if ( config().host().empty() )
     {
         std::cerr << config().teamName()
                   << " coach: ***ERROR*** coach: server host name is empty"
                   << std::endl;
-        return;
+        M_client->setServerAlive( false );
+        return false;
     }
 
     // just create a connection. init command is automaticcaly sent
     // by BasicClient's run() method.
-    if ( ! connectTo( config().host().c_str(),
-                      config().port(),
-                      static_cast< long >( config().intervalMSec() ) ) )
+    if ( ! M_client->connectTo( config().host().c_str(),
+                                config().port(),
+                                static_cast< long >( config().intervalMSec() ) ) )
     {
         std::cerr << config().teamName()
                   << " coach: ***ERROR*** Failed to connect." << std::endl;
-        setServerAlive( false );
-    }
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
-*/
-void
-CoachAgent::sendInitCommand()
-{
-    if ( ! isServerAlive() )
-    {
-        std::cerr << config().teamName()
-                  << " coach: server is not alive" << std::endl;
-        return;
+        M_client->setServerAlive( false );
     }
 
-    // make command string
-    bool success = true;
-    if ( config().useCoachName()
-         && ! config().coachName().empty() )
-    {
-        CoachInitCommand com( config().teamName(),
-                              config().version(),
-                              config().coachName() );
-        success = sendCommand( com );
-    }
-    else
-    {
-        CoachInitCommand com( config().teamName(),
-                              config().version() );
-        success = sendCommand( com );
-    }
-
-    if ( ! success )
-    {
-        std::cerr << config().teamName()
-                  << " coach: Failed to init coach...\nExit ..."
-                  << std::endl;
-        setServerAlive( false );
-    }
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
-*/
-void
-CoachAgent::sendByeCommand()
-{
-    CoachByeCommand com;
-    sendCommand( com );
-    setServerAlive( false );
+    sendInitCommand();
+    return true;
 }
 
 /*-------------------------------------------------------------------*/
@@ -330,14 +289,21 @@ CoachAgent::sendByeCommand()
 void
 CoachAgent::handleMessage()
 {
+    if ( ! M_client )
+    {
+        std::cerr << "CoachAgent::handleMessage(). Client is not registered."
+                  << std::endl;
+        return;
+    }
+
     int counter = 0;
     GameTime start_time = M_impl->current_time_;
 
     // receive and analyze message
-    while ( recvMessage() > 0 )
+    while ( M_client->recvMessage() > 0 )
     {
         ++counter;
-        parse( recvBuf() );
+        parse( M_client->message() );
     }
 
     if ( M_impl->current_time_.cycle() > start_time.cycle() + 1
@@ -371,27 +337,35 @@ CoachAgent::handleMessage()
 
 */
 void
-CoachAgent::handleTimeout()
+CoachAgent::handleTimeout( const int timeout_count,
+                           const int waited_msec )
 {
-    if ( waitedMSec() > config().serverWaitSeconds() * 1000 )
+    if ( ! M_client )
+    {
+        std::cerr << "CoachAgent::handleTimeout(). Client is not registered."
+                  << std::endl;
+        return;
+    }
+
+    if ( waited_msec > config().serverWaitSeconds() * 1000 )
     {
         if ( config().useEye() )
         {
             std::cout << config().teamName()
                       << " coach: waited "
-                      << waitedMSec() / 1000
+                      << waited_msec / 1000
                       << " seconds. server down??" << std::endl;
-            setServerAlive( false );
+            M_client->setServerAlive( false );
             return;
         }
 
-        if ( waitedMSec() > config().serverWaitSeconds() * 2 * 1000 )
+        if ( waited_msec > config().serverWaitSeconds() * 2 * 1000 )
         {
             std::cout << config().teamName()
                       << " coach: waited "
-                      << waitedMSec() / 1000
+                      << waited_msec / 1000
                       << " seconds. server down??" << std::endl;
-            setServerAlive( false );
+            M_client->setServerAlive( false );
             return;
         }
 
@@ -408,7 +382,17 @@ CoachAgent::handleTimeout()
 void
 CoachAgent::handleExit()
 {
-    if ( isServerAlive() )
+    finalize();
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+CoachAgent::finalize()
+{
+    if ( M_client->isServerAlive() )
     {
         sendByeCommand();
     }
@@ -425,7 +409,7 @@ CoachAgent::action()
 {
     if ( M_impl->last_decision_time_ != M_impl->current_time_ )
     {
-        doAction();
+        actionImpl();
         M_impl->last_decision_time_ = M_impl->current_time_;
     }
 
@@ -525,7 +509,7 @@ CoachAgent::analyzeInit( const char * msg )
 
     if ( std::sscanf( msg, "(init %c ok)", &side ) != 1 )
     {
-        setServerAlive( false );
+        M_client->setServerAlive( false );
         return;
     }
 
@@ -536,7 +520,7 @@ CoachAgent::analyzeInit( const char * msg )
                   << world().time()
                   << " received unexpected init message. " << msg
                   << std::endl;
-        setServerAlive( false );
+        M_client->setServerAlive( false );
         return;
     }
 
@@ -779,9 +763,9 @@ CoachAgent::analyzeServerParam( const char * msg )
 
     if ( ServerParam::i().slowDownFactor() != 1 )
     {
-        long interval = ( ServerParam::i().simulatorStep()
+        long interval = ( config().intervalMSec()
                           * ServerParam::i().slowDownFactor() );
-        setIntervalMSec( interval );
+        M_client->setIntervalMSec( interval );
     }
 
     M_worldmodel.initFreeformCount();
@@ -842,8 +826,8 @@ CoachAgent::analyzeOK( const char * msg )
             std::cout << config().teamName()
                       << " coach: "
                       << world().time()
-                      << " recv " << msg << std::endl;
-            //setCompressionLevel( level );
+                      << " set compression level " << level<< std::endl;
+            M_client->setCompressionLevel( level );
         }
     }
     else if ( ! std::strncmp( msg, "(ok eye ", std::strlen( "(ok eye " ) ) )
@@ -939,7 +923,59 @@ CoachAgent::sendCommand( const CoachCommand & com )
         return false;
     }
 
-    return ( sendMessage( str.c_str() ) > 0 );
+    return ( M_client->sendMessage( str.c_str() ) > 0 );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+CoachAgent::sendInitCommand()
+{
+    if ( ! M_client->isServerAlive() )
+    {
+        std::cerr << config().teamName()
+                  << " coach: server is not alive" << std::endl;
+        return;
+    }
+
+    // make command string
+    bool success = true;
+    if ( config().useCoachName()
+         && ! config().coachName().empty() )
+    {
+        CoachInitCommand com( config().teamName(),
+                              config().version(),
+                              config().coachName() );
+        success = sendCommand( com );
+    }
+    else
+    {
+        CoachInitCommand com( config().teamName(),
+                              config().version() );
+        success = sendCommand( com );
+    }
+
+    if ( ! success )
+    {
+        std::cerr << config().teamName()
+                  << " coach: Failed to init coach...\nExit ..."
+                  << std::endl;
+        M_client->setServerAlive( false );
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+CoachAgent::sendByeCommand()
+{
+    CoachByeCommand com;
+    sendCommand( com );
+    M_client->setServerAlive( false );
 }
 
 /*-------------------------------------------------------------------*/
@@ -1080,7 +1116,7 @@ CoachAgent::doSayFreeform( const std::string & msg )
     std::ostringstream ostr;
     ostr << "(say (freeform \"" << msg << "\"))";
 
-    return ( sendMessage( ostr.str().c_str() ) > 0 );
+    return ( M_client->sendMessage( ostr.str().c_str() ) > 0 );
 }
 
 }
