@@ -158,8 +158,8 @@ GlobalVisualSensor::parseV7( const char * msg )
 
     msg += n_read;
 
-    std::string teamname_left;
-    std::string teamname_right;
+    std::string team_name_left;
+    std::string team_name_right;
 
     // ((p "TEAM" UNUM[ goalie]) <x> <y> <vx> <vy> <body> <neck>[ <arm>][ t])
     while ( *msg != '\0' )
@@ -186,22 +186,22 @@ GlobalVisualSensor::parseV7( const char * msg )
         }
 
         SideID side = NEUTRAL;
-        if ( teamname_left == teamname )
+        if ( team_name_left == teamname )
         {
             side = LEFT;
         }
-        else if ( teamname_right == teamname )
+        else if ( team_name_right == teamname )
         {
             side = RIGHT;
         }
-        else if ( teamname_left.empty() )
+        else if ( team_name_left.empty() )
         {
-            teamname_left = teamname;
+            team_name_left = teamname;
             side = LEFT;
         }
-        else if ( teamname_right.empty() )
+        else if ( team_name_right.empty() )
         {
-            teamname_right = teamname;
+            team_name_right = teamname;
             side = RIGHT;
         }
         else
@@ -262,6 +262,18 @@ GlobalVisualSensor::parseV7( const char * msg )
         while ( *msg != '\0' && *msg != ')' ) ++msg;
         while ( *msg == ')' ) ++msg;
     }
+
+    if ( M_team_name_left.empty()
+         && team_name_left.length() > 2 )
+    {
+        M_team_name_left = team_name_left.substr( 1, team_name_left.length() - 2 );
+    }
+
+    if ( M_team_name_right.empty()
+         && team_name_right.length() > 2 )
+    {
+        M_team_name_right = team_name_right.substr( 1, team_name_right.length() - 2 );
+    }
 }
 
 /*-------------------------------------------------------------------*/
@@ -329,8 +341,8 @@ GlobalVisualSensor::parseV6( const char * msg )
 
     msg += n_read;
 
-    std::string teamname_left;
-    std::string teamname_right;
+    std::string team_name_left;
+    std::string team_name_right;
 
     // ((player "TEAM" UNUM[ goalie]) <x> <y> <body> <neck> <vx> <vy>)
     while ( *msg != '\0' )
@@ -357,22 +369,22 @@ GlobalVisualSensor::parseV6( const char * msg )
         }
 
         SideID side = NEUTRAL;
-        if ( teamname_left == teamname )
+        if ( team_name_left == teamname )
         {
             side = LEFT;
         }
-        else if ( teamname_right == teamname )
+        else if ( team_name_right == teamname )
         {
             side = RIGHT;
         }
-        else if ( teamname_left.empty() )
+        else if ( team_name_left.empty() )
         {
-            teamname_left = teamname;
+            team_name_left = teamname;
             side = LEFT;
         }
-        else if ( teamname_right.empty() )
+        else if ( team_name_right.empty() )
         {
-            teamname_right = teamname;
+            team_name_right = teamname;
             side = RIGHT;
         }
         else
@@ -417,6 +429,16 @@ GlobalVisualSensor::parseV6( const char * msg )
         while ( *msg != '\0' && *msg != ')' ) ++msg;
         while ( *msg == ')' ) ++msg;
     }
+
+    if ( ! team_name_left.empty() )
+    {
+        M_team_name_left = team_name_left;
+    }
+
+    if ( ! team_name_right.empty() )
+    {
+        M_team_name_right = team_name_right;
+    }
 }
 
 /*-------------------------------------------------------------------*/
