@@ -46,7 +46,7 @@ namespace {
 
 /*-------------------------------------------------------------------*/
 /*!
-  get squared value
+  \brief get squared value
   \param val input value
   \return squared value
  */
@@ -59,7 +59,7 @@ SQUARE( const double & val )
 
 /*-------------------------------------------------------------------*/
 /*!
-  solve quadratic fomula
+  \brief solve quadratic fomula
   \param a fomula constant A
   \param b fomula constant B
   \param c fomula constant C
@@ -139,10 +139,12 @@ Circle2D::intersection( const Line2D & line,
         if ( n_sol > 0 )
         {
             double y1 = -line.c() / line.b();
+
             if ( sol1 )
             {
                 sol1->assign( x1, y1 );
             }
+
             if ( n_sol > 1 && sol2 )
             {
                 sol2->assign( x2, y1 );
@@ -174,10 +176,12 @@ Circle2D::intersection( const Line2D & line,
         {
             sol1->assign( line.getX( y1 ), y1 );
         }
+
         if ( n_sol > 1 && sol2 )
         {
             sol2->assign( line.getX( y2 ), y2 );
         }
+
         return n_sol;
     }
 }
@@ -209,12 +213,12 @@ Circle2D::intersection( const Ray2D & ray,
         --n_sol;
     }
 
-    if ( n_sol > 0 )
+    if ( n_sol > 0 && sol1 )
     {
         *sol1 = tsol1;
     }
 
-    if ( n_sol > 1 )
+    if ( n_sol > 1 && sol2 )
     {
         *sol2 = tsol2;
     }
@@ -222,6 +226,41 @@ Circle2D::intersection( const Ray2D & ray,
     return n_sol;
 }
 
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+int
+Circle2D::intersection( const Circle2D & circle,
+                        Vector2D * sol1,
+                        Vector2D * sol2 ) const
+{
+    double rel_x = circle.center().x - this->center().x;
+    double rel_y = circle.center().y - this->center().y;
+
+    double center_dist2 = rel_x * rel_x + rel_y * rel_y;
+    double center_dist = std::sqrt( center_dist2 );
+
+    if ( center_dist < std::fabs( this->radius() - circle.radius() )
+         || this->radius() + circle.radius() < center_dist )
+    {
+        return 0;
+    }
+
+    //std::cerr << "must exist intersection C1: " << this->center() << this->radius()
+    //        << " C2: " << circle.center() << circle.radius()
+    //        << std::endl;
+    // line that passes through the intersection points
+    Line2D line( -2.0 * rel_x,
+                 -2.0 * rel_y,
+                 circle.center().r2()
+                 - circle.radius() * circle.radius()
+                 - this->center().r2()
+                 + this->radius() * this->radius() );
+
+    return this->intersection( line, sol1, sol2 );
+}
+
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -232,6 +271,7 @@ Circle2D::circumcircle( const Vector2D & a,
                         const Vector2D & c )
 {
     Vector2D center = Triangle2D::circumcenter( a, b, c );
+
     if ( ! center.valid() )
     {
         std::cerr << "Circle2D::circumcircle()"
