@@ -489,9 +489,11 @@ public:
     /*!
       \brief output parameter usage by command line option style
       \param os reference to output streamf
+      \param with_default if true, default value is printed.
       \return reference to output stream
     */
-    std::ostream & printHelp( std::ostream & os ) const;
+    std::ostream & printHelp( std::ostream & os,
+                              const bool with_default = true ) const;
 
     /*!
       \brief output parameter name and value
