@@ -51,11 +51,8 @@ public:
     typedef std::map< char, int > CharToIntCont;
     typedef std::vector< char > IntToCharCont;
 
-    enum {
-        POSX_BIT_L5 = 8, //!< used by posToLongL5
-        POSY_BIT_L5 = 8, //!< used by posToLongL5
-        VEL_BIT_L5 = 6, //!< used by posToLongL5
-    };
+    //! constant error value (= std::numeric_limits< double >::max())
+    static const double ERROR_VALUE;
 
     //! x normalize factor (field length) to limit inputed x
     static const double X_NORM_FACTOR;
@@ -64,6 +61,9 @@ public:
     //! speed normalize factor to limit inputed speed range
     static const double SPEED_NORM_FACTOR;
 
+    static const int POSX_BIT_L5; //!< used by posToLongL5
+    static const int POSY_BIT_L5; //!< used by posToLongL5
+    static const int VEL_BIT_L5; //!< used by posToLongL5
     static const boost::int32_t POSX_MASK_L5; //!< used by longToPosL5
     static const boost::int32_t POSY_MASK_L5; //!< used by longToPosL5
     static const boost::int32_t VELX_MASK_L5; //!< used by longToPosL5
@@ -97,8 +97,8 @@ private:
       \param vel velocity to be converted
       \return converted integer
     */
-    boost::int32_t posToLongL5( const Vector2D & pos,
-                                const Vector2D & vel ) const;
+    boost::int32_t posVelToInt32( const Vector2D & pos,
+                                  const Vector2D & vel ) const;
 
     /*!
       \brief decode 32bits integer to position and velocity
@@ -106,9 +106,9 @@ private:
       \param pos variable pointer to store the converted position
       \param vel variable pointer to store the converted velocity
     */
-    void longToPosL5( const boost::int32_t & val,
-                      Vector2D * pos,
-                      Vector2D * vel ) const;
+    void int32ToPosVel( const boost::int32_t & val,
+                        Vector2D * pos,
+                        Vector2D * vel ) const;
 
 public:
 
@@ -132,17 +132,15 @@ public:
           return M_int_to_char_map;
       }
     /*
-      \brief encode position and velocity to  5 characters.
+      \brief encode position and velocity to 5 characters.
       \param pos position value to be encoded
       \param vel velocity value to be encoded
-      \param to reference to the variable to store the encoded result
       \return true if successfully encoded.
 
       The length of result string must be 5.
     */
-    bool encodeL5( const Vector2D & pos,
-                   const Vector2D & vel,
-                   std::string & to ) const;
+    std::string encodePosVelToStr5( const Vector2D & pos,
+                                    const Vector2D & vel ) const;
 
     /*!
       \brief decode 5 characters to position and velocity
@@ -153,17 +151,16 @@ public:
 
       The length of 'from' must be 5.
     */
-    bool decodeL5( const std::string & from,
-                   Vector2D * pos,
-                   Vector2D * vel ) const;
+    bool decodeStr5ToPosVel( const std::string & from,
+                             Vector2D * pos,
+                             Vector2D * vel ) const;
 
     /*!
       \brief encode coordinate value( x or y ) to 2 characters.
       \param xy coordinate value to be encoded, X or Y.
       This value should be within [-norm_factor, norm_factor]
       \param norm_factor normalize factor for xy
-      \param to reference to the variable to store the encoded result
-      \return true if successfully encoded.
+      \return result string
 
       Following condition must be satisfied:
       - (norm_factor*2)/COORD_STEP_L2 < CHARSIZE^2
@@ -172,9 +169,8 @@ public:
       norm_factor will be changed depending on the type of target
       coordinate value, i.e. field length or field width.
     */
-    bool encodeCoordL2( const double & xy,
-                        const double & norm_factor,
-                        std::string & to ) const;
+    std::string encodeCoordToStr2( const double & xy,
+                                   const double & norm_factor ) const;
 
     /*!
       \brief decode 2 characters to coordinate value( x or y )
@@ -186,43 +182,56 @@ public:
       norm_factor will be changed depending on the type of target
       coordinate value, i.e. field length or field width.
     */
-    double decodeCoordL2( const char ch1,
-                          const char ch2,
-                          const double & norm_factor ) const;
+    double decodeStr2ToCoord( const char ch1,
+                              const char ch2,
+                              const double & norm_factor ) const;
+
+    /*!
+      \brief encode position value to 4 characters with 0.1 step
+      \param pos position value to be encoded
+      \return encoded string instance. if encoding is failed, empty string
+      is returned
+     */
+    std::string encodePosToStr4( const Vector2D & pos ) const;
+
+    /*!
+      \brief decode 4 characters to position value
+      \param from message string to be decoded
+      \return decoded position value
+     */
+    Vector2D decodeStr4ToPos( const std::string & from ) const;
+
 
     /*!
       \brief encode position value to 7 characters
       \param pos position to be encoded
-      \param msg reference to the variable to store the encoded result
-      \return true if successfully encoded
+      \return encoded string instance. if encoding is failed, empty string
+      is returned
     */
     std::string encodePosToStr7( const Vector2D & pos ) const;
 
     /*!
       \brief decode 7 characters to position value
       \param msg head of message to be decoded
-      \param pos variable pointer to store the decoded result
-      \return true if successfully decoded
+      \return decoded position value
     */
     Vector2D decodeStr7ToPos( const char * msg ) const;
 
     /*!
       \brief encode speed value to 1 character.
       \param xy velocity element to be encoded, X or Y
-      \param to reference to the variable to store the encoded result
-      \return true if successfully encoded.
+      \return result char
 
-      xy value is normalized by SPEED_NORM_FACTOR
+      speed value is normalized by SPEED_NORM_FACTOR
     */
-    bool encodeSpeedL1( const double & xy,
-                        std::string & to ) const;
+    char encodeSpeedToChar( const double & xy ) const;
 
     /*!
       \brief decode 1 character to speed value
       \param ch character to be decoded
       \return decoded speed value. if failed to decode, HUGE_VAL is returned
     */
-    double decodeSpeedL1( const char ch ) const;
+    double decodeCharToSpeed( const char ch ) const;
 
     /*!
       \brief encode position and velocity to 6 charecters.
@@ -230,8 +239,8 @@ public:
       \param vel velocity to be encoded
       \return encoded string. if encode is failed, return the empty string
     */
-    std::string encodeL6( const Vector2D & pos,
-                          const Vector2D & vel ) const;
+    std::string encodePosVelToStr6( const Vector2D & pos,
+                                    const Vector2D & vel ) const;
 
     /*!
       \brief decode 6 characters to position and velocity
@@ -240,9 +249,9 @@ public:
       \param vel variable pointer to store the decoded velocity value
       \return true if successfully decoded
     */
-    bool decodeL6( const std::string & from,
-                   Vector2D * pos,
-                   Vector2D * vel ) const;
+    bool decodeStr6ToPosVel( const std::string & from,
+                             Vector2D * pos,
+                             Vector2D * vel ) const;
 
     /*!
       \brief convert decimal uniform number to hex number
