@@ -36,6 +36,7 @@
 #include "audio_sensor.h"
 #include "logger.h"
 
+#include <rcsc/common/server_param.h>
 #include <rcsc/math_util.h>
 
 #include <cstdio>
@@ -55,7 +56,7 @@ AudioSensor::AudioSensor()
     , M_ball_vel( 0.0, 0.0 )
     , M_receiver_number( 0 )
     , M_receive_pos( 50.0, 0.0 )
-    , M_goalie_number( 0 )
+    , M_goalie_number( Unum_Unknown )
     , M_goalie_pos( 50.0, 0.0 )
     , M_offside_line_x( 100000.0 )
     , M_defense_line_x( -100000.0 )
@@ -139,9 +140,10 @@ AudioSensor::parsePlayer( const char * msg,
     M_player_message.time_ = current;
     M_player_message.side_ = NEUTRAL;
     M_player_message.unum_ = unum;
-    M_player_message.type_ = PlayerMessage::ILLEGAL;
     M_player_message.dir_ = dir;
 
+    M_player_message.info_.clear();
+
     parseTeammateAudioV8();
 }
 
@@ -314,230 +316,85 @@ AudioSensor::parseTrainer( const char * msg,
 /*!
 
  */
-bool
+void
 AudioSensor::parseTeammateAudioV8()
 {
     if ( M_player_message.message_.empty() )
     {
-        return false;
+        return;
     }
 
     const char * msg = M_player_message.message_.c_str();
 
-    // ball info
-    if ( ! std::strncmp( msg, "b ", 2 ) )
+    while ( *msg != '\0' )
     {
-        if ( M_player_message.message_.length() != 8 )
+        int len = 0;
+        PlayerMessage::Info info;
+
+        switch ( *msg ) {
+        case 'a': // attack request
+            len = 1;
+            info.type_ = PlayerMessage::ATTACK_REQUEST;
+            break;
+        case 'b': // ball info
+            len = parseBallInfo( msg );
+            if ( len == 0 ) break;
+            info.type_ = PlayerMessage::BALL_INFO;
+            break;
+        case 'd': // defense line
+            len = parseDefenseLineInfo( msg );
+            if ( len == 0 ) break;
+            info.type_ = PlayerMessage::DEFENSE_LINE_INFO;
+            break;
+        case 'g': // goalie info
+            len = parseGoalieInfo( msg );
+            if ( len == 0 ) break;
+            info.type_ = PlayerMessage::OPPONENT_GOALIE_INFO;
+            break;
+        case 'h': // hey pass info
+            len = parseHeyPassInfo( msg );
+            if ( len == 0 ) break;
+            info.type_ = PlayerMessage::HEY_PASS;
+            break;
+        case 'i': // intercept info
+            len = parseInterceptInfo( msg );
+            if ( len == 0 ) break;
+            info.type_ = PlayerMessage::INTERCEPT_INFO;
+            break;
+        case 'o': // offside line
+            len = parseOffsideLineInfo( msg );
+            if ( len == 0 ) break;
+            info.type_ = PlayerMessage::OFFSIDE_LINE_INFO;
+            break;
+        case 'p': // pass info
+            len = parsePassInfo( msg );
+            if ( len == 0 ) break;
+            info.type_ = PlayerMessage::PASS_INFO;
+            break;
+        case 'w': // wait request
+            len = 1;
+            info.type_ = PlayerMessage::WAIT_REQUEST;
+            break;
+        default:
+            break;
+        }
+
+        if ( len == 0 )
         {
-            std::cerr << "***ERROR*** AudioSensor::parseTeammateAudioV8"
-                      << " Illega ball info [" << msg << "]"
-                      << std::endl;
             dlog.addText( Logger::SENSOR,
-                          "AudioSensor: Illegal ball info [%s]",
-                          msg );
-            return false;
+                          "AudioSensor: unsupported message? [%s] in [%s]",
+                          msg, M_player_message.message_.c_str() );
+            // assign the rest string
+            M_player_message.message_ = msg;
+            return;
         }
 
-        std::string ball_str = M_player_message.message_.substr( 2, 6 );
-        if ( ! M_codec.decodeL6( ball_str, &M_ball_pos, &M_ball_vel ) )
-        {
-            std::cerr << "***ERROR*** AudioSensor::parseTeammateAudioV8"
-                      << " Failed to decode ball [" << msg
-                      << "]  str=["  << ball_str << "]"
-                      << std::endl;
-            dlog.addText( Logger::SENSOR,
-                          "AudioSensor: Failed to decode Ball Info [%s]",
-                          msg );
+        info.str_.assign( msg, 0, len );
+        M_player_message.info_.push_back( info );
 
-            return false;
-        }
-
-        M_player_message.type_ = PlayerMessage::BALL_INFO;
-        return true;
-    }
-    ////////////////////////////////////////////////////////
-    // pass info
-    if ( ! std::strncmp( msg, "p ", 2 ) )
-    {
-        if ( parsePassInfo( msg ) )
-        {
-            M_player_message.type_ = PlayerMessage::PASS_INFO;
-            return true;
-        }
-        std::cerr  << "***ERROR*** AudioSensor::parseTeammateAudioV8"
-                   << " Failed to decode pass [" << msg << "]"
-                   << std::endl;
-        dlog.addText( Logger::SENSOR,
-                      "AudioSensor: Failed to decode Pass Info [%s]",
-                      msg );
-        return false;
+        msg += len;
     }
 
-    ////////////////////////////////////////////////////////
-    // goalie info
-    if ( ! std::strncmp( msg, "g ", 2 ) )
-    {
-        if ( parseGoalieInfo( msg ) )
-        {
-            M_player_message.type_ = PlayerMessage::OPPONENT_GOALIE_INFO;
-            return true;
-        }
-        std::cerr  << "***ERROR*** AudioSensor::parseTeammateAudioV8"
-                   << " Failed to decode goalie [" << msg << "]"
-                   << std::endl;
-        dlog.addText( Logger::SENSOR,
-                      "AudioSensor: Failed to decode Goalie Info [%s]",
-                      msg );
-        return false;
-    }
-
-    ////////////////////////////////////////////////////////
-    // offside line
-    if ( ! std::strncmp( msg, "o ", 2 ) )
-    {
-        if ( parseOffsideLineInfo( msg ) )
-        {
-            M_player_message.type_ = PlayerMessage::OFFSIDE_LINE_INFO;
-            return true;
-        }
-        std::cerr  << "***ERROR*** AudioSensor::parseTeammateAudioV8"
-                   << " Failed to decode offside line [" << msg << "]"
-                   << std::endl;
-        dlog.addText( Logger::SENSOR,
-                      "AudioSensor: Failed to decode Offside Line Info [%s]",
-                      msg );
-        return false;
-    }
-    ////////////////////////////////////////////////////////
-    // defense line
-    if ( ! std::strncmp( msg, "d ", 2 ) )
-    {
-        if ( parseDefenseLineInfo( msg ) )
-        {
-            M_player_message.type_ = PlayerMessage::DEFENSE_LINE_INFO;
-            return true;
-        }
-        std::cerr  << "***ERROR*** AudioSensor::parseTeammateAudioV8"
-                   << " Failed to decode defense line [" << msg << "]"
-                   << std::endl;
-        dlog.addText( Logger::SENSOR,
-                      "AudioSensor: Failed to decode Defense Line Info [%s]",
-                      msg );
-        return false;
-    }
-
-    ////////////////////////////////////////////////////////
-    // wait request
-    if ( ! std::strncmp( msg, "w", 1 ) )
-    {
-        M_player_message.type_ = PlayerMessage::WAIT_REQUEST;
-        return true;
-    }
-
-    ////////////////////////////////////////////////////////
-    // attack request
-    if ( ! std::strncmp( msg, "a", 1 ) )
-    {
-        M_player_message.type_ = PlayerMessage::ATTACK_REQUEST;
-        return true;
-    }
-
-    ////////////////////////////////////////////////////////
-    // intercept info
-    if ( ! std::strncmp( msg, "i ", 2 ) )
-    {
-        if ( parseInterceptInfo( msg ) )
-        {
-            M_player_message.type_ = PlayerMessage::INTERCEPT;
-            return true;
-        }
-        std::cerr  << "***ERROR*** AudioSensor::parseTeammateAudioV8"
-                   << " Failed to decode intercept info [" << msg << "]"
-                   << std::endl;
-        dlog.addText( Logger::SENSOR,
-                      "AudioSensor: Failed to decode intercept info [%s]",
-                      msg );
-        return false;
-    }
-
-    ////////////////////////////////////////////////////////
-    // hey pass info
-    if ( ! std::strncmp( msg, "h ", 2 ) )
-    {
-        M_player_message.type_ = PlayerMessage::HEY_PASS;
-        return true;
-    }
-
-
-    //std::cerr << "***ERROR*** AudioSensor::parseTeammateAudioV8"
-    //          << " Failed to decode ["
-    //          << raw_message << "]" << std::endl;
-    dlog.addText( Logger::SENSOR,
-                  "AudioSensor: unsupported message [%s]",
-                  M_player_message.message_.c_str() );
-
-    return false;
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
- */
-bool
-AudioSensor::parsePassInfo( const char * msg )
-{
-    if ( *msg != 'p' )
-    {
-        return false;
-    }
-
-    // skip space
-    msg += 2;
-
-    ///////////////////////////////////////
-    // read recever number
-    switch ( *msg ) {
-    case '1': M_receiver_number = 1; break;
-    case '2': M_receiver_number = 2; break;
-    case '3': M_receiver_number = 3; break;
-    case '4': M_receiver_number = 4; break;
-    case '5': M_receiver_number = 5; break;
-    case '6': M_receiver_number = 6; break;
-    case '7': M_receiver_number = 7; break;
-    case '8': M_receiver_number = 8; break;
-    case '9': M_receiver_number = 9; break;
-    case 'A': M_receiver_number = 10; break;
-    case 'B': M_receiver_number = 11; break;
-    default:
-        std::cerr << "AudioSensor:: Unexpected receiver number " << msg
-                  << std::endl;
-        return false;
-    }
-
-    // skip space
-    msg += 2;
-
-    ///////////////////////////////////////
-    // read x coordinate
-    int x = 0, y = 0;
-
-    x += 100 * (*msg - '0'); ++msg;
-    x += 10 * (*msg - '0'); ++msg;
-    x += (*msg - '0'); ++msg;
-
-    y += 100 * (*msg - '0'); ++msg;
-    y += 10 * (*msg - '0'); ++msg;
-    y += (*msg - '0'); ++msg;
-
-    M_receive_pos.x = static_cast< double >( x ) * 0.5 - 52.5;
-    M_receive_pos.y = static_cast< double >( y ) * 0.5 - 34.0;
-
-
-    dlog.addText( Logger::SENSOR,
-                  "AudioSensor: success!  receiver= %d  receive_pos(%f %f)",
-                  M_receiver_number,
-                  M_receive_pos.x, M_receive_pos.y );
-    return true;
 }
 
 /*-------------------------------------------------------------------*/
@@ -549,16 +406,62 @@ AudioSensor::encodeBall( const Vector2D & ball_pos,
                          const Vector2D & ball_vel,
                          std::string & say_buf ) const
 {
-    say_buf = "b ";
-    std::string msg = M_codec.encodeL6( ball_pos, ball_vel );
+    if ( (int)say_buf.length() + 6 > ServerParam::i().playerSayMsgSize() )
+    {
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor::encodeBallInfo(). over the message size = %d",
+                      say_buf.length() );
+        return;
+    }
+
+    std::string msg = codec().encodePosVelToStr6( ball_pos, ball_vel );
     if ( msg.empty() )
     {
-        say_buf.erase();
+        return;
     }
-    else
+
+    say_buf += 'b';
+    say_buf += msg;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+int
+AudioSensor::parseBallInfo( const char * msg )
+{
+    if ( std::strlen( msg ) < 7 )
     {
-        say_buf += msg;
+        std::cerr << "***ERROR*** AudioSensor::parseTeammateAudioV8"
+                  << " Illega ball info [" << msg << "]"
+                  << std::endl;
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor: Illegal ball info [%s]",
+                      msg );
+        return 0;
     }
+
+    std::string ball_str( msg, 1, 6 );
+
+    if ( ! codec().decodeStr6ToPosVel( ball_str,
+                                       &M_ball_pos, &M_ball_vel ) )
+    {
+        std::cerr << "***ERROR*** AudioSensor::parseTeammateAudioV8"
+                  << " Failed to decode ball [" << msg
+                  << "]" << std::endl;
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor: Failed to decode Ball Info [%s]",
+                      msg );
+        return 0;
+    }
+
+    dlog.addText( Logger::SENSOR,
+                  "AudioSensor::parseBallInfo() success! pos(%.1f %.1f) vel(%.1f %.1f)",
+                  M_ball_pos.x, M_ball_pos.y,
+                  M_ball_vel.x, M_ball_vel.y );
+
+    return 7;
 }
 
 /*-------------------------------------------------------------------*/
@@ -570,51 +473,90 @@ AudioSensor::encodePass( const int receiver,
                          const Vector2D & receive_pos,
                          std::string & say_buf ) const
 {
-    say_buf = "p ";
-
-    switch ( receiver ) {
-    case 1: say_buf += "1 "; break;
-    case 2: say_buf += "2 "; break;
-    case 3: say_buf += "3 "; break;
-    case 4: say_buf += "4 "; break;
-    case 5: say_buf += "5 "; break;
-    case 6: say_buf += "6 "; break;
-    case 7: say_buf += "7 "; break;
-    case 8: say_buf += "8 "; break;
-    case 9: say_buf += "9 "; break;
-    case 10: say_buf += "A "; break;
-    case 11: say_buf += "B "; break;
-    default:
-        std::cerr << "AudioSensor::encode Unexpected receiver number "
-                  << receiver << std::endl;
-        say_buf.erase();
+    if ( (int)say_buf.length() + 6 > ServerParam::i().playerSayMsgSize() )
+    {
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor::encodePassInfo(). over the message size = %d",
+                      say_buf.length() );
         return;
     }
 
-    double tmp = min_max( -52.5, receive_pos.x, 52.5 );
-    tmp += 52.5;
-    tmp *= 2.0;
+    char hex = AudioCodec::unum2hex( receiver );
+    if ( hex == '\0' )
+    {
+        std::cerr << "AudioSensor::encodePass. "
+                  << "Unexpected receiver number "
+                  << receiver << std::endl;
+        return;
+    }
 
-    int ix = static_cast< int >( rint( tmp ) );
-
-    tmp = min_max( -34.0, receive_pos.y, 34.0 );
-    tmp += 34.0;
-    tmp *= 2.0;
-
-    int iy = static_cast< int >( rint( tmp ) );
-
-
-    char xymsg[8];
-    std::snprintf( xymsg, 8, "%03d%03d", ix, iy );
-    if ( std::strlen( xymsg ) != 6 )
+    std::string pos_str = codec().encodePosToStr4( receive_pos );
+    if ( pos_str.empty() )
     {
         std::cerr << "AudioSensor::encode XY error "
-                  << receive_pos << " -> " << xymsg << std::endl;
-        say_buf.erase();
+                  << receive_pos << " -> " << pos_str
+                  << std::endl;
         return;
     }
 
-    say_buf += xymsg;
+    say_buf += 'p';
+    say_buf += hex;
+    say_buf += pos_str;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+int
+AudioSensor::parsePassInfo( const char * msg )
+{
+    if ( std::strlen( msg ) < 6
+         || msg[0] != 'p' )
+    {
+        std::cerr << "AudioSensor::parsePassInfo. "
+                  << "Unexpected pass message ["
+                  << msg << "] len = " << std::strlen( msg )
+                  << std::endl;
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor: Failed to decode Pass Info [%s]",
+                      msg );
+        return 0;
+    }
+    ++msg;
+
+    M_receiver_number = AudioCodec::hex2unum( *msg );
+    if ( M_receiver_number == Unum_Unknown )
+    {
+        std::cerr << "AudioSensor::parsePassInfo. "
+                  << "Unexpected receiver number. "
+                  << msg
+                  << std::endl;
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor: Failed to decode Pass Info [%s]",
+                      msg );
+        return 0;
+    }
+    ++msg;
+
+    M_receive_pos = codec().decodeStr4ToPos( msg );
+
+    if ( ! M_receive_pos )
+    {
+        std::cerr << "AudioSensor::parsePassInfo. "
+                  << "Unexpected value " << msg
+                  << std::endl;
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor: Failed to decode Pass Info [%s]",
+                      msg );
+        return 0;
+    }
+
+    dlog.addText( Logger::SENSOR,
+                  "AudioSensor::parsePassInfo() success! receiver= %d  receive_pos(%f %f)",
+                  M_receiver_number,
+                  M_receive_pos.x, M_receive_pos.y );
+    return 6;
 }
 
 /*-------------------------------------------------------------------*/
@@ -626,7 +568,13 @@ AudioSensor::encodeGoalie( const int goalie_number,
                            const Vector2D & goalie_pos,
                            std::string & say_buf ) const
 {
-    say_buf = "g ";
+    if ( (int)say_buf.length() + 6 > ServerParam::i().playerSayMsgSize() )
+    {
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor::encodeGoalieInfo(). over the message size = %d",
+                      say_buf.length() );
+        return;
+    }
 
     char hex = AudioCodec::unum2hex( goalie_number );
     if ( hex == '\0' )
@@ -634,22 +582,21 @@ AudioSensor::encodeGoalie( const int goalie_number,
         std::cerr << "AudioSensor::encodeGoalieInfo. "
                   << "Unexpected goalie number "
                   << goalie_number << std::endl;
-        say_buf.erase();
         return;
     }
 
-    say_buf += hex;
-
-    std::string pos_str = codec().encodePosToStr7( goalie_pos );
-    if ( pos_str.length() != 7 )
+    std::string pos_str = codec().encodePosToStr4( goalie_pos );
+    if ( pos_str.empty() )
     {
         std::cerr << "AudioSensor::encodeGoalieInfo. "
                   << "Failed to encode goalie info "
                   << "( " << goalie_pos.x << goalie_pos.y << ")"
                   << std::endl;
-        say_buf.erase();
+        return;
     }
 
+    say_buf += 'g';
+    say_buf += hex;
     say_buf += pos_str;
 }
 
@@ -657,48 +604,53 @@ AudioSensor::encodeGoalie( const int goalie_number,
 /*!
 
  */
-bool
+int
 AudioSensor::parseGoalieInfo( const char * msg )
 {
-    if ( std::strlen( msg ) != 10
-         || msg[0] != 'g'
-         || msg[1] != ' ' )
+    if ( std::strlen( msg ) < 6
+         || msg[0] != 'g' )
     {
         std::cerr << "AudioSensor::parseGoalieInfo. "
                   << "Unexpected goalie message ["
                   << msg << "] len = " << std::strlen( msg )
                   << std::endl;
-        return false;
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor: Failed to decode Goalie Info [%s]",
+                      msg );
+        return 0;
     }
-    msg += 2;
+    ++msg;
 
-    int unum = AudioCodec::hex2unum( *msg );
-    if ( unum == 0 )
+    M_goalie_number = AudioCodec::hex2unum( *msg );
+    if ( M_goalie_number == Unum_Unknown )
     {
         std::cerr << "AudioSensor::parseGoalieInfo. "
                   << "Unexpected goalie number " << msg
                   << std::endl;
-        return false;
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor: Failed to decode Goalie Info [%s]",
+                      msg );
+        return 0;
     }
-
     ++msg;
 
-    M_goalie_pos = codec().decodeStr7ToPos( msg );
+    M_goalie_pos = codec().decodeStr4ToPos( msg );
 
     if ( ! M_goalie_pos )
     {
         std::cerr << "AudioSensor::parseGoalieInfo. "
-                  << " Failed to decode goalie potiiton"
+                  << "Unexpected value " << msg
                   << std::endl;
         dlog.addText( Logger::SENSOR,
-                      "AudioSensor: Failed to decode goalie potiiton" );
-        return false;
+                      "AudioSensor: Failed to decode Goalie Info [%s]",
+                      msg );
+        return 0;
     }
 
     dlog.addText( Logger::SENSOR,
-                  "AudioSensor: success!  goalie pos = (%.2f %.2f)",
+                  "AudioSensor::parseGoalieInfo() success! goalie pos = (%.2f %.2f)",
                   M_goalie_pos.x, M_goalie_pos.y );
-    return true;
+    return 6;
 }
 
 /*-------------------------------------------------------------------*/
@@ -709,44 +661,68 @@ void
 AudioSensor::encodeOffsideLineX( const double & offside_line_x,
                                  std::string & say_buf ) const
 {
-    char buf[10];
-    std::snprintf( buf, 10, "o %.1f", offside_line_x );
-    say_buf = buf;
+    if ( (int)say_buf.length() + 3 > ServerParam::i().playerSayMsgSize() )
+    {
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor::encodeOffsideLineX(). over the message size = %d",
+                      say_buf.length() );
+        return;
+    }
+
+    std::string msg = codec().encodeCoordToStr2( offside_line_x,
+                                                 AudioCodec::X_NORM_FACTOR );
+    if ( msg.empty() )
+    {
+        std::cerr << "AudioSensor::encodeOffsideLineX. "
+                  << " Failed to encode. value = " << offside_line_x
+                  << std::endl;
+    }
+
+    say_buf += 'o';
+    say_buf += msg;
 }
 
 /*-------------------------------------------------------------------*/
 /*!
-  "o <x>"
+
  */
-bool
+int
 AudioSensor::parseOffsideLineInfo( const char * msg )
 {
-    if ( msg[0] != 'o'
-         || msg[1] != ' ' )
+    if ( std::strlen( msg ) < 3
+         || msg[0] != 'o' )
     {
         std::cerr << "AudioSensor::parseOffsideLineInfo. "
-                  << "Unexpected value message ["
+                  << " Unexpected message ["
                   << msg << "]"
                   << std::endl;
-        return false;
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor: Failed to decode Offside Line Info [%s]",
+                      msg );
+        return 0;
     }
-    msg += 2;
+    ++msg;
 
-
-    double val = std::strtod( msg, NULL );
-    if ( val == -HUGE_VAL
-         || val == HUGE_VAL )
+    double val = codec().decodeStr2ToCoord( msg[0], msg[1],
+                                            AudioCodec::X_NORM_FACTOR );
+    if ( val == AudioCodec::ERROR_VALUE )
     {
         std::cerr << "AudioSensor::parseOffsideLineInfo. "
                   << " Failed to read offside line"
                   << std::endl;
-        return false;
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor: Failed to decode Offside Line Info [%s]",
+                      msg );
+        return 0;
     }
 
-
     M_offside_line_x = val;
 
-    return true;
+    dlog.addText( Logger::SENSOR,
+                  "AudioSensor::parseOffsideLineInfo() success! x=%.1f",
+                  val );
+
+    return 3;
 }
 
 /*-------------------------------------------------------------------*/
@@ -757,44 +733,68 @@ void
 AudioSensor::encodeDefenseLineX( const double & defense_line_x,
                                  std::string & say_buf ) const
 {
-    char buf[10];
-    std::snprintf( buf, 10, "d %.1f", defense_line_x );
-    say_buf = buf;
+    if ( (int)say_buf.length() + 3 > ServerParam::i().playerSayMsgSize() )
+    {
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor::encodeBallInfo(). over the message size = %d",
+                      say_buf.length() );
+        return;
+    }
+
+    std::string msg = codec().encodeCoordToStr2( defense_line_x,
+                                                 AudioCodec::X_NORM_FACTOR );
+    if ( msg.empty() )
+    {
+        std::cerr << "AudioSensor::encodeDefenseLineX. "
+                  << " Failed to encode. value = " << defense_line_x
+                  << std::endl;
+    }
+
+    say_buf += 'd';
+    say_buf += msg;
 }
 
 /*-------------------------------------------------------------------*/
 /*!
-  "d <x>"
+
  */
-bool
+int
 AudioSensor::parseDefenseLineInfo( const char * msg )
 {
-    if ( msg[0] != 'd'
-         || msg[1] != ' ' )
+    if ( std::strlen( msg ) < 3
+         || msg[0] != 'd' )
     {
         std::cerr << "AudioSensor::parseDefenseLineInfo. "
-                  << "Unexpected value message ["
+                  << " Unexpected message ["
                   << msg << "]"
                   << std::endl;
-        return false;
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor: Failed to decode Defense Line Info [%s]",
+                      msg );
+        return 0;
     }
-    msg += 2;
+    ++msg;
 
-
-    double val = std::strtod( msg, NULL );
-    if ( val == -HUGE_VAL
-         || val == HUGE_VAL )
+    double val = codec().decodeStr2ToCoord( msg[0], msg[1],
+                                            AudioCodec::X_NORM_FACTOR );
+    if ( val == AudioCodec::ERROR_VALUE )
     {
         std::cerr << "AudioSensor::parseDefenseLineInfo. "
-                  << " Failed to read defense line"
+                  << " Failed to read offside line"
                   << std::endl;
-        return false;
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor: Failed to decode Defense Line Info [%s]",
+                      msg );
+        return 0;
     }
 
-
     M_defense_line_x = val;
 
-    return true;
+    dlog.addText( Logger::SENSOR,
+                  "AudioSensor::parseDefenseLineInfo() success! x=%.1f",
+                  val );
+
+    return 3;
 }
 
 /*-------------------------------------------------------------------*/
@@ -806,45 +806,98 @@ AudioSensor::encodeInterceptInfo( const int unum,
                                   const int cycle,
                                   std::string & say_buf ) const
 {
-    char buf[10];
-    std::snprintf( buf, 10, "i %d %d", unum, cycle );
-    say_buf = buf;
+    if ( (int)say_buf.length() + 3 > ServerParam::i().playerSayMsgSize() )
+    {
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor::encodeInterceptInfo(). over the message size = %d",
+                      say_buf.length() );
+        return;
+    }
+
+    char hex = AudioCodec::unum2hex( unum );
+    if ( hex == '\0' )
+    {
+        std::cerr << "AudioSensor::encodeInterceptInfo. "
+                  << "Unexpected number "
+                  << unum << std::endl;
+        return;
+    }
+
+    try
+    {
+        char ch = codec().intToCharMap().at( cycle );
+
+        say_buf += 'i';
+        say_buf += hex;
+        say_buf += ch;
+    }
+    catch ( ... )
+    {
+        std::cerr << "AudioSensor::encodeInterceptInfo."
+                  << " Exception caught! Failed to encode cycle = "
+                  << cycle
+                  << std::endl;
+    }
 }
 
 /*-------------------------------------------------------------------*/
 /*!
 
  */
-bool
+int
 AudioSensor::parseInterceptInfo( const char * msg )
 {
-    if ( msg[0] != 'i'
-         || msg[1] != ' ' )
+    if ( std::strlen( msg ) < 3
+         || msg[0] != 'i' )
     {
         std::cerr << "AudioSensor::parseInterceptInfo. "
-                  << "Unexpected message Id ["
+                  << " Unexpected message = ["
                   << msg << "]"
                   << std::endl;
-        return false;
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor: Failed to decode intercept info [%s]",
+                      msg );
+        return 0;
     }
+    ++msg;
 
-    int unum = 0;
-    int cycle = 0;
-
-    if ( std::sscanf( msg, "i %d %d", &unum , &cycle ) != 2 )
+    int number = AudioCodec::hex2unum( *msg );
+    if ( number == Unum_Unknown )
     {
         std::cerr << "AudioSensor::parseInterceptInfo. "
-                  << "Unexpected value message(2) ["
+                  << " Invalid player number. message = ["
                   << msg << "]"
                   << std::endl;
-        return false;
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor: Failed to decode intercept info [%s]",
+                      msg );
+        return 0;
+    }
+    ++msg;
+
+    AudioCodec::CharToIntCont::const_iterator cycle
+        = codec().charToIntMap().find( *msg );
+    if ( cycle == codec().charToIntMap().end() )
+    {
+        std::cerr << "AudioSensor::parseInterceptInfo. "
+                  << " Invalid cycle. message = ["
+                  << msg << "]"
+                  << std::endl;
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor: Failed to decode intercept info [%s]",
+                      msg );
+        return 0;
     }
 
     M_intercept_time = M_time;
-    M_intercept_number = unum;
-    M_intercept_cycle = cycle;
+    M_intercept_number = number;
+    M_intercept_cycle = cycle->second;
 
-    return true;
+    dlog.addText( Logger::SENSOR,
+                  "AudioSensor::parseInterceptInfo() success! unum=%d cycle=%d",
+                  number, cycle->second );
+
+    return 3;
 }
 
 /*-------------------------------------------------------------------*/
@@ -856,26 +909,43 @@ AudioSensor::encodeHeyPassInfo( const int unum,
                                 const Vector2D & pos,
                                 std::string & say_buf ) const
 {
-    char hex = AudioCodec::unum2hex( unum );
-    if ( hex == '\0' )
+    if ( (int)say_buf.length() + 6 > ServerParam::i().playerSayMsgSize() )
     {
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor::encodeHeyPassInfo(). over the message size = %d",
+                      say_buf.length() );
         return;
     }
 
-    say_buf = "i ";
-    say_buf += hex;
-
-    std::string pos_str = codec().encodePosToStr7( pos );
-    if ( pos_str.length() != 7 )
+    char hex = AudioCodec::unum2hex( unum );
+    if ( hex == '\0' )
     {
         std::cerr << "AudioSensor::encodeHeyPassInfo. "
-                  << "Failed to encode hey passe info. unum="
+                  << "Failed to encode hey pass info. unum="
                   << unum
                   << " ( " << pos.x << pos.y << ")"
                   << std::endl;
-        say_buf.erase();
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor: Failed to encode hey pass info" );
+        return;
     }
 
+    std::string pos_str = codec().encodePosToStr4( pos );
+    if ( pos_str.length() != 4 )
+    {
+        std::cerr << "AudioSensor::encodeHeyPassInfo. "
+                  << "Failed to encode hey pass info. unum="
+                  << unum
+                  << " ( " << pos.x << pos.y << ")"
+                  << std::endl;
+        dlog.addText( Logger::SENSOR,
+                      "AudioSensor: Failed to encode hey pass info" );
+        say_buf.erase();
+        return;
+    }
+
+    say_buf += 'h';
+    say_buf += hex;
     say_buf += pos_str;
 }
 
@@ -883,50 +953,50 @@ AudioSensor::encodeHeyPassInfo( const int unum,
 /*!
 
  */
-bool
+int
 AudioSensor::parseHeyPassInfo( const char * msg )
 {
-    if ( std::strlen( msg ) != 10
-         || msg[0] != 'h'
-         || msg[1] != ' ' )
+    if ( std::strlen( msg ) < 6
+         || msg[0] != 'h' )
     {
         std::cerr << "AudioSensor::parseHeyPassInfo. "
                   << "Unexpected hey pass message ["
                   << msg << "] len = " << std::strlen( msg )
                   << std::endl;
-        return false;
+        return 0;
     }
-    msg += 2;
+    ++msg;
 
-    M_hey_pass_number = AudioCodec::hex2unum( *msg );
-    if ( M_hey_pass_number == 0 )
+    int number = AudioCodec::hex2unum( *msg );
+    if ( number == Unum_Unknown )
     {
         std::cerr << "AudioSensor::parseGoalieInfo. "
                   << "Unexpected goalie number " << msg
                   << std::endl;
-        return false;
+        return 0;
     }
-
     ++msg;
 
-    M_hey_pass_pos = codec().decodeStr7ToPos( msg );
-
-    if ( ! M_hey_pass_pos )
+    Vector2D pos = codec().decodeStr4ToPos( msg );
+    if ( ! pos )
     {
         std::cerr << "AudioSensor::parseHeyPassInfo. "
                   << " Failed to decode hey pass potiiton"
                   << std::endl;
         dlog.addText( Logger::SENSOR,
                       "AudioSensor: Failed to decode hey pass potiiton" );
-        return false;
+        return 0;
     }
 
+    M_hey_pass_number = number;
+    M_hey_pass_pos = pos;
+    M_hey_pass_time = M_time;
+
     dlog.addText( Logger::SENSOR,
-                  "AudioSensor: success!  hey pass pos = (%.2f %.2f)",
+                  "AudioSensor::parseHeyPassInfo() success! hey pass pos = (%.2f %.2f)",
                   M_hey_pass_pos.x, M_hey_pass_pos.y );
 
-    M_hey_pass_time = M_time;
-    return true;
+    return 0;
 }
 
 }
