@@ -48,9 +48,9 @@ class SelfObject;
 
   order of update process.
   update()
-  -> updateByCollision()
   -> updateAll() (updatePos(),updateOnlyVel(), updateOnlyRelativePos())
   -> updateByHeardInfo()
+  -> updateByCollision()
   -> updateSelfRelated()
 */
 class BallObject {
@@ -259,6 +259,36 @@ public:
           return M_lost_count;
       }
 
+    /*!
+      \brief get heard time
+      \return game time value
+     */
+    const
+    GameTime & heardTime() const
+      {
+          return M_heard_time;
+      }
+
+    /*!
+      \brief get heard position
+      \return position value
+     */
+    const
+    Vector2D & heardPos() const
+      {
+          return M_heard_pos;
+      }
+
+    /*!
+      \brief get heard velocity
+      \return velocity value
+     */
+    const
+    Vector2D & heardVel() const
+      {
+          return M_heard_vel;
+      }
+
     /*!
       \brief velify global position accuracy
       \return true if position has enough accuracy
@@ -328,8 +358,11 @@ public:
       Thie method is called when collision estimatation
     */
     void updateByCollision( const Vector2D & pos,
+                            const int pos_count,
                             const Vector2D & rpos,
-                            const Vector2D & vel );
+                            const int rpos_count,
+                            const Vector2D & vel,
+                            const int vel_count );
 
     /*!
       \brief update relative position using see info.
