@@ -40,7 +40,7 @@
 #include "player_object.h"
 #include "logger.h"
 
-#include <rcsc/param/server_param.h>
+#include <rcsc/common/server_param.h>
 #include <rcsc/game_time.h>
 
 #include <algorithm>
@@ -109,6 +109,9 @@ InterceptTable::update()
     }
     s_update_time = M_world.time();
 
+    dlog.addText( Logger::INTERCEPT,
+                  "InterceptTable update" );
+
     // clear all data
     this->clear();
 
@@ -193,6 +196,36 @@ InterceptTable::update()
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+void
+InterceptTable::updateTeammate( const int unum,
+                                const int cycle )
+{
+    if ( M_fastest_teammate
+         && cycle >= M_teammate_reach_cycle )
+    {
+        return;
+    }
+
+    const PlayerObject * p = M_world.getTeammate( unum );
+    if ( p )
+    {
+        M_fastest_teammate = p;
+        M_teammate_reach_cycle = cycle;
+
+        dlog.addText( Logger::INTERCEPT,
+                      "<-----By Heard: Intercept Teammate  fastest reach step = %d."
+                      " teammate %d (%.1f %.1f)",
+                      M_teammate_reach_cycle,
+                      M_fastest_teammate->unum(),
+                      M_fastest_teammate->pos().x,
+                      M_fastest_teammate->pos().y );
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 void
 InterceptTable::createBallCache()
@@ -328,20 +361,19 @@ InterceptTable::predictTeammate()
                           (*it)->posCount() );
             continue;
         }
-        HeteroID id = Hetero_Default;
-        if ( (*it)->unum() > 0 ) id = M_world.getTeammateHeteroID( (*it)->unum() );
-        const PlayerType * player_type = PlayerTypeSet::i().get( id );
+
+        const PlayerType * player_type = M_world.teammatePlayerType( (*it)->unum() );
         if ( ! player_type )
         {
             std::cerr << M_world.time()
                       << " " << __FILE__ << ":" << __LINE__
                       << "  Failed to get teammate player type. unum = "
-                      << (*it)->unum() << " id = " << id
+                      << (*it)->unum()
                       << std::endl;
             dlog.addText( Logger::INTERCEPT,
                           "ERROR. Intercept. Failed to get teammate player type.",
-                          " unum = %d  id = %d",
-                          (*it)->unum(), id );
+                          " unum = %d",
+                          (*it)->unum() );
             continue;
         }
 
@@ -402,20 +434,19 @@ InterceptTable::predictOpponent()
                           (*it)->posCount() );
             continue;
         }
-        HeteroID id = Hetero_Unknown;
-        if ( (*it)->unum() > 0 ) id = M_world.getOpponentHeteroID( (*it)->unum() );
-        const PlayerType * player_type = PlayerTypeSet::i().get( id );
+
+        const PlayerType * player_type = M_world.opponentPlayerType( (*it)->unum() );
         if ( ! player_type )
         {
             std::cerr << M_world.time()
                       << " " << __FILE__ << ":" << __LINE__
                       << "  Failed to get opponent player type. unum = "
-                      << (*it)->unum() << " id = " << id
+                      << (*it)->unum()
                       << std::endl;
             dlog.addText( Logger::INTERCEPT,
                           "ERROR. Intercept Failed to get opponent player type."
-                          " unum = %d  id = %d",
-                          (*it)->unum(), id );
+                          " unum = %d",
+                          (*it)->unum());
             continue;
         }
         int cycle = predictor.predict( *(*it), *player_type,
