@@ -35,8 +35,8 @@
 
 #include "interception.h"
 
-#include <rcsc/param/server_param.h>
-#include <rcsc/param/player_type.h>
+#include <rcsc/common/server_param.h>
+#include <rcsc/common/player_type.h>
 #include <rcsc/soccer_math.h>
 #include <rcsc/math_util.h>
 
