@@ -949,9 +949,7 @@ Localization::~Localization()
 
 */
 void
-Localization::localizeSelf( const VisualSensor::LineCont & lines,
-                            const VisualSensor::MarkerCont & markers,
-                            const VisualSensor::MarkerCont & behind_markers,
+Localization::localizeSelf( const VisualSensor & see,
                             double * self_face,
                             double * self_face_err,
                             Vector2D * self_pos,
@@ -972,10 +970,10 @@ Localization::localizeSelf( const VisualSensor::LineCont & lines,
 
     ////////////////////////////////////////////////////////////////////
     // get face dir & face error
-    *self_face = M_impl->getFaceDirByLines( lines );
+    *self_face = M_impl->getFaceDirByLines( see.lines() );
     if ( *self_face == VisualSensor::DIR_ERR )
     {
-        *self_face = M_impl->getFaceDirByMarkers( markers );
+        *self_face = M_impl->getFaceDirByMarkers( see.markers() );
         if ( *self_face == VisualSensor::DIR_ERR )
         {
             dlog.addText( Logger::WORLD,
@@ -992,7 +990,7 @@ Localization::localizeSelf( const VisualSensor::LineCont & lines,
 
     ////////////////////////////////////////////////////////////////////
     // if no marker, we cannot estimate my position
-    if ( markers.empty() )
+    if ( see.markers().empty() )
     {
         dlog.addText( Logger::WORLD,
                       "localizer.self. no marker!" );
@@ -1001,8 +999,10 @@ Localization::localizeSelf( const VisualSensor::LineCont & lines,
 
     ////////////////////////////////////////////////////////////////////
     // generate particles using the nearest marker
-    M_impl->generateParticles( markers.front(), markers.front().id_,
-                               *self_face, *self_face_err );
+    M_impl->generateParticles( see.markers().front(),
+                               see.markers().front().id_,
+                               *self_face,
+                               *self_face_err );
 
 
     if ( M_impl->particles().empty() )
@@ -1014,14 +1014,16 @@ Localization::localizeSelf( const VisualSensor::LineCont & lines,
 
     ////////////////////////////////////////////////////////////////////
     // update particles by known markers
-    M_impl->updateParticlesByMarkers( markers, *self_face, *self_face_err );
+    M_impl->updateParticlesByMarkers( see.markers(),
+                                      *self_face,
+                                      *self_face_err );
 
     // in order to estimate the Id of nearest behind marker,
     // it is necessary to calculate current estimation result,
     M_impl->getAverageOfParticles( self_pos, self_pos_err );
 
     // update particles by nearest behind marker
-    M_impl->updateParticlesByBehindMarker( behind_markers,
+    M_impl->updateParticlesByBehindMarker( see.behindMarkers(),
                                            *self_pos, *self_face, *self_face_err );
     if ( M_impl->particles().empty() )
     {
@@ -1040,7 +1042,7 @@ Localization::localizeSelf( const VisualSensor::LineCont & lines,
 
 */
 void
-Localization::localizeBallRelative( const VisualSensor::BallCont & balls,
+Localization::localizeBallRelative( const VisualSensor & see,
                                     const double & self_face,
                                     const double & self_face_err,
                                     Vector2D * rpos,
@@ -1048,12 +1050,12 @@ Localization::localizeBallRelative( const VisualSensor::BallCont & balls,
                                     Vector2D * rvel,
                                     Vector2D * rvel_err ) const
 {
-    if ( balls.empty() )
+    if ( see.balls().empty() )
     {
         return;
     }
 
-    const VisualSensor::BallT & ball = balls.front();
+    const VisualSensor::BallT & ball = see.balls().front();
 
     ////////////////////////////////////////////////////////////////////
     // get polar range info
