@@ -36,7 +36,7 @@
 #include <rcsc/player/stamina_model.h>
 #include <rcsc/player/fullstate_sensor.h>
 
-#include <rcsc/param/player_type.h>
+#include <rcsc/common/player_type.h>
 #include <rcsc/geom/vector_2d.h>
 #include <rcsc/game_time.h>
 #include <rcsc/types.h>
