@@ -226,7 +226,7 @@ public:
         */
         PlayerT()
             : MovableT()
-            , unum_( -1 )
+            , unum_( Unum_Unknown )
             , goalie_( false )
             , body_( VisualSensor::DIR_ERR )
             , face_( VisualSensor::DIR_ERR )
@@ -239,7 +239,7 @@ public:
         void reset()
           {
               MovableT::reset();
-              unum_ = -1;
+              unum_ = Unum_Unknown;
               goalie_ = false;
               body_ = VisualSensor::DIR_ERR;
               face_ = VisualSensor::DIR_ERR;
