@@ -39,9 +39,11 @@
 
 #include <rcsc/coach/global_visual_sensor.h>
 
-#include <rcsc/param/server_param.h>
-#include <rcsc/param/player_param.h>
-#include <rcsc/param/player_type.h>
+#include <rcsc/common/basic_client.h>
+#include <rcsc/common/server_param.h>
+#include <rcsc/common/player_param.h>
+#include <rcsc/common/player_type.h>
+
 #include <rcsc/param/param_map.h>
 #include <rcsc/param/conf_file_parser.h>
 #include <rcsc/param/cmd_line_parser.h>
@@ -169,7 +171,7 @@ TrainerAgentImpl::updateServerStatus()
 
 */
 TrainerAgent::TrainerAgent()
-    : BasicClient()
+    : SoccerAgent()
     , M_impl( new TrainerAgentImpl )
     , M_config()
 {
@@ -201,7 +203,7 @@ TrainerAgent::visualSensor() const
 
 */
 bool
-TrainerAgent::doInit( CmdLineParser & cmd_parser )
+TrainerAgent::initImpl( CmdLineParser & cmd_parser )
 {
     bool help = false;
     std::string trainer_config_file;
@@ -220,7 +222,7 @@ TrainerAgent::doInit( CmdLineParser & cmd_parser )
     {
         system_param_map.printHelp( std::cout );
         trainer_param_map.printHelp( std::cout );
-        setServerAlive( false );
+        M_client->setServerAlive( false );
         return false;
     }
 
@@ -247,61 +249,35 @@ TrainerAgent::doInit( CmdLineParser & cmd_parser )
 /*!
 
 */
-void
-TrainerAgent::doConnect()
+bool
+TrainerAgent::handleStart()
 {
+    if ( ! M_client )
+    {
+        return false;
+    }
+
     if ( config().host().empty() )
     {
         std::cerr << "trainer: ***ERROR*** server host name is empty"
                   << std::endl;
-        return;
+        M_client->setServerAlive( false );
+        return false;
     }
 
     // just create a connection. init command is automaticcaly sent
     // by BasicClient's run() method.
-    if ( ! connectTo( config().host().c_str(),
-                      config().port(),
-                      static_cast< long >( ServerParam::i().simulatorStep() ) ) )
+    if ( ! M_client->connectTo( config().host().c_str(),
+                                config().port(),
+                                static_cast< long >( ServerParam::i().simulatorStep() ) ) )
     {
         std::cerr << "trainer: ***ERROR*** failed to connect." << std::endl;
-        setServerAlive( false );
-    }
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
-*/
-void
-TrainerAgent::sendInitCommand()
-{
-    if ( ! isServerAlive() )
-    {
-        std::cerr << "server is not alive" << std::endl;
-        return;
+        M_client->setServerAlive( false );
+        return false;
     }
 
-    // make command string
-    TrainerInitCommand com( config().version() );
-    if ( ! sendCommand( com ) )
-    {
-        std::cerr << "trainer: Failed to init...\nExit." << std::endl;
-        setServerAlive( false );
-        return;
-    }
-
-    std::cerr << "trainer: send init" << std::endl;
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
-*/
-void
-TrainerAgent::sendByeCommand()
-{
-    // trainer has no bye type command
-    setServerAlive( false );
+    sendInitCommand();
+    return true;
 }
 
 /*-------------------------------------------------------------------*/
@@ -311,11 +287,18 @@ TrainerAgent::sendByeCommand()
 void
 TrainerAgent::handleMessage()
 {
+    if ( ! M_client )
+    {
+        std::cerr << "TrainerAgent::handleMessage(). Client is not registered."
+                  << std::endl;
+        return;
+    }
+
     // start parse process
 
-    while ( recvMessage() > 0 )
+    while ( M_client->recvMessage() > 0 )
     {
-        parse( recvBuf() );
+        parse( M_client->message() );
     }
 
     if ( M_impl->think_received_ )
@@ -337,11 +320,19 @@ TrainerAgent::handleMessage()
 
 */
 void
-TrainerAgent::handleTimeout()
+TrainerAgent::handleTimeout( const int timeout_count,
+                             const int waited_msec )
 {
-    if ( waitedMSec() > config().serverWaitSeconds() * 1000 )
+    if ( ! M_client )
     {
-        setServerAlive( false );
+        std::cerr << "TrainerAgent::handleTimeout(). Client is not registered."
+                  << std::endl;
+        return;
+    }
+
+    if ( waited_msec > config().serverWaitSeconds() * 1000 )
+    {
+        M_client->setServerAlive( false );
     }
 }
 
@@ -352,7 +343,17 @@ TrainerAgent::handleTimeout()
 void
 TrainerAgent::handleExit()
 {
-    if ( isServerAlive() )
+    finalize();
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+TrainerAgent::finalize()
+{
+    if ( M_client->isServerAlive() )
     {
         sendByeCommand();
     }
@@ -422,7 +423,7 @@ TrainerAgent::parse( const char * msg )
     {
         std::cerr << "trainer:"
                   << M_impl->current_time_
-                  << " received nsupported Message : ["
+                  << " received unsupported Message : ["
                   << msg << "]" << std::endl;
     }
 }
@@ -439,7 +440,7 @@ TrainerAgent::analyzeInit( const char * msg )
         std::cerr << "trainer: Failed to init trainer.. init reply message=["
                   << msg << ']'
                   << std::endl;
-        setServerAlive( false );
+        M_client->setServerAlive( false );
         return;
     }
 
@@ -454,7 +455,7 @@ TrainerAgent::analyzeInit( const char * msg )
         doEar( true );
     }
 
-    setIntervalMSec( ServerParam::i().simulatorStep() );
+    M_client->setIntervalMSec( ServerParam::i().simulatorStep() );
 }
 
 /*-------------------------------------------------------------------*/
@@ -689,7 +690,7 @@ TrainerAgent::analyzeServerParam( const char * msg )
     {
         long interval = ( ServerParam::i().simulatorStep()
                           * ServerParam::i().slowDownFactor() );
-        setIntervalMSec( interval );
+        M_client->setIntervalMSec( interval );
     }
 }
 
@@ -723,7 +724,7 @@ TrainerAgent::analyzeOK( const char * msg )
             std::cerr << "trainer: "
                       << M_impl->current_time_
                       << " set compression level " << level << std::endl;
-            // setCompressionLevel( level );
+            M_client->setCompressionLevel( level );
         }
     }
     else if ( ! std::strncmp( msg, "(ok eye ", std::strlen( "(ok eye " ) ) )
@@ -744,6 +745,7 @@ TrainerAgent::analyzeOK( const char * msg )
         std::cout << "trainer: "
                   << M_impl->current_time_
                   << " recv " << msg << std::endl;
+        analyzeTeamNames( msg );
     }
     else
     {
@@ -818,7 +820,50 @@ TrainerAgent::sendCommand( const TrainerCommand & com )
         return false;
     }
 
-    return ( sendMessage( str.c_str() ) > 0 );
+    if ( M_client->sendMessage( str.c_str() ) > 0 )
+    {
+        std::cout << "OK send command [" << str << "]" << std::endl;
+        return true;
+    }
+
+    std::cout << "failed to send command [" << str << "]" << std::endl;
+    return false;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+TrainerAgent::sendInitCommand()
+{
+    if ( ! M_client->isServerAlive() )
+    {
+        std::cerr << "trainer: server is not alive" << std::endl;
+        return;
+    }
+
+    // make command string
+    TrainerInitCommand com( config().version() );
+    if ( ! sendCommand( com ) )
+    {
+        std::cerr << "trainer: Failed to init...\nExit." << std::endl;
+        M_client->setServerAlive( false );
+        return;
+    }
+
+    std::cerr << "trainer: send init" << std::endl;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+TrainerAgent::sendByeCommand()
+{
+    // trainer has no bye type command
+    M_client->setServerAlive( false );
 }
 
 /*-------------------------------------------------------------------*/
@@ -981,7 +1026,7 @@ TrainerAgent::action()
 {
     if ( M_impl->last_decision_time_ != M_impl->current_time_ )
     {
-        doAction();
+        actionImpl();
         M_impl->last_decision_time_ = M_impl->current_time_;
     }
 
