@@ -1,5 +1,13 @@
 // -*-c++-*-
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#ifdef HAVE_NETINET_IN_H
+#include <netinet/in.h>
+#endif
+
 #include <cmath>
 #include <cstring>
 #include <iostream>
@@ -45,31 +53,6 @@
 */
 
 
-inline
-int
-nstohi( const short value )
-{
-    return static_cast< int >( static_cast< short >( ntohs( value ) ) );
-}
-
-
-inline
-double
-nstohd( const short value )
-{
-    return static_cast< double >( static_cast< short >( ntohs( value ) ) )
-        / rcsc::rcg::SHOWINFO_SCALE;
-}
-
-inline
-double
-nltohd( const long & value )
-{
-    return static_cast< double >( static_cast< long >( ntohl( value ) ) )
-        / rcsc::rcg::SHOWINFO_SCALE2;
-}
-
-
 class TextPrinter
     : public rcsc::rcg::Handler {
 private:
@@ -95,14 +78,14 @@ private:
           { }
         void update( const rcsc::rcg::player_t & player )
           {
-              kick_ = nstohi( player.kick_count );
-              dash_ = nstohi( player.dash_count );
-              turn_ = nstohi( player.turn_count );
-              say_ = nstohi( player.say_count );
-              turn_neck_ = nstohi( player.turn_neck_count );
-              catch_ = nstohi( player.catch_count );
-              move_ = nstohi( player.move_count );
-              change_view_ = nstohi( player.change_view_count );
+              kick_ = rcsc::rcg::nstohi( player.kick_count );
+              dash_ = rcsc::rcg::nstohi( player.dash_count );
+              turn_ = rcsc::rcg::nstohi( player.turn_count );
+              say_ = rcsc::rcg::nstohi( player.say_count );
+              turn_neck_ = rcsc::rcg::nstohi( player.turn_neck_count );
+              catch_ = rcsc::rcg::nstohi( player.catch_count );
+              move_ = rcsc::rcg::nstohi( player.move_count );
+              change_view_ = rcsc::rcg::nstohi( player.change_view_count );
           }
     };
 
@@ -339,7 +322,7 @@ TextPrinter::handlePlayMode( char playmode )
 */
 bool
 TextPrinter::handleTeamInfo( const rcsc::rcg::team_t& team_left,
-                               const rcsc::rcg::team_t& team_right )
+                             const rcsc::rcg::team_t& team_right )
 {
     if ( M_left_team_name.empty() )
     {
@@ -381,22 +364,22 @@ TextPrinter::handleServerParam( const rcsc::rcg::server_params_t & param )
 {
     M_init_written = true;
     M_os << "(Init"
-         << " (goal_width " << nltohd( param.goal_width ) << ")"
-         << " (player_size " << nltohd( param.player_size ) << ")"
-         << " (ball_size " << nltohd( param.ball_size ) << ")"
-         << " (kickable_margin " << nltohd( param.kickable_margin ) << ")"
-         << " (visible_distance " <<  nltohd( param.visible_distance ) << ")"
-         << " (kickable_area " << ( nltohd( param.player_size )
-                                    + nltohd( param.ball_size )
-                                    + nltohd( param.kickable_margin ) )
+         << " (goal_width " << rcsc::rcg::nltohd( param.goal_width ) << ")"
+         << " (player_size " << rcsc::rcg::nltohd( param.player_size ) << ")"
+         << " (ball_size " << rcsc::rcg::nltohd( param.ball_size ) << ")"
+         << " (kickable_margin " << rcsc::rcg::nltohd( param.kickable_margin ) << ")"
+         << " (visible_distance " <<  rcsc::rcg::nltohd( param.visible_distance ) << ")"
+         << " (kickable_area " << ( rcsc::rcg::nltohd( param.player_size )
+                                    + rcsc::rcg::nltohd( param.ball_size )
+                                    + rcsc::rcg::nltohd( param.kickable_margin ) )
          << ")"
-         << " (catchable_area_l " << nltohd( param.catch_area_l ) << ")"
-         << " (catchable_area_w " << nltohd( param.catch_area_w ) << ")"
+         << " (catchable_area_l " << rcsc::rcg::nltohd( param.catch_area_l ) << ")"
+         << " (catchable_area_w " << rcsc::rcg::nltohd( param.catch_area_w ) << ")"
          << " (half_time " << static_cast< short >( ntohs( param.half_time ) ) << ")"
-         << " (ckick_margin " << nltohd( param.corner_kick_margin ) << ")"
-         << " (offside_active_area_size " << nltohd( param.offside_active_area ) << ")"
-         << " (offside_kick_margin " << nltohd( param.offside_kick_margin ) << ")"
-         << " (audio_cut_dist " << nltohd( param.audio_cut_dist ) << ")"
+         << " (ckick_margin " << rcsc::rcg::nltohd( param.corner_kick_margin ) << ")"
+         << " (offside_active_area_size " << rcsc::rcg::nltohd( param.offside_active_area ) << ")"
+         << " (offside_kick_margin " << rcsc::rcg::nltohd( param.offside_kick_margin ) << ")"
+         << " (audio_cut_dist " << rcsc::rcg::nltohd( param.audio_cut_dist ) << ")"
          << ")"
          << std::endl;
     return true;
@@ -525,8 +508,8 @@ TextPrinter::printBall( std::ostream & os,
                         const rcsc::rcg::pos_t & ball ) const
 {
     os << "(ball "
-       << nstohd( ball.x ) << " "
-       << nstohd( ball.y ) << ")";
+       << rcsc::rcg::nstohd( ball.x ) << " "
+       << rcsc::rcg::nstohd( ball.y ) << ")";
 
     return os;
 }
@@ -540,10 +523,10 @@ TextPrinter::printBall( std::ostream & os,
                         const rcsc::rcg::ball_t & ball ) const
 {
     os << "(ball "
-       << nltohd( ball.x ) << " "
-       << nltohd( ball.y ) << " "
-       << nltohd( ball.deltax ) << " "
-       << nltohd( ball.deltay ) << ")";
+       << rcsc::rcg::nltohd( ball.x ) << " "
+       << rcsc::rcg::nltohd( ball.y ) << " "
+       << rcsc::rcg::nltohd( ball.deltax ) << " "
+       << rcsc::rcg::nltohd( ball.deltay ) << ")";
 
     return os;
 }
@@ -571,8 +554,8 @@ TextPrinter::printPlayer( std::ostream & os,
         os << " g";
     }
     os << " (position "
-       << nstohd( player.x ) << " "
-       << nstohd( player.y ) << " "
+       << rcsc::rcg::nstohd( player.x ) << " "
+       << rcsc::rcg::nstohd( player.y ) << " "
        << static_cast< double >( static_cast< short >( ntohs( player.angle ) ) )
        << ")";
 
@@ -623,10 +606,10 @@ TextPrinter::printPlayer( std::ostream & os,
     const short mode = ntohs( player.mode );
 
     double body_deg
-        = nltohd( player.body_angle )
+        = rcsc::rcg::nltohd( player.body_angle )
         * (180.0 / M_PI);
     double head_deg
-        = nltohd( player.head_angle )
+        = rcsc::rcg::nltohd( player.head_angle )
         * (180.0 / M_PI);
     head_deg += body_deg;
     while ( head_deg > 180.0 )
@@ -648,22 +631,22 @@ TextPrinter::printPlayer( std::ostream & os,
         os << " g";
     }
     os << " (position "
-       << nltohd( player.x ) << " "
-       << nltohd( player.y ) << " "
-       << nltohd( player.deltax ) << " "
-       << nltohd( player.deltay ) << " "
+       << rcsc::rcg::nltohd( player.x ) << " "
+       << rcsc::rcg::nltohd( player.y ) << " "
+       << rcsc::rcg::nltohd( player.deltax ) << " "
+       << rcsc::rcg::nltohd( player.deltay ) << " "
        << body_deg << " "
        << head_deg << ")"; // global neck angle
     os << " (stamina "
-       << nltohd( player.stamina ) << ")";
+       << rcsc::rcg::nltohd( player.stamina ) << ")";
 
     ///////////////////////////////////////////////////////
 
-    if ( count.turn_ != nstohi( player.turn_count ) )
+    if ( count.turn_ != rcsc::rcg::nstohi( player.turn_count ) )
     {
         os << " (turn)";
     }
-    else if ( count.dash_ != nstohi( player.dash_count ) )
+    else if ( count.dash_ != rcsc::rcg::nstohi( player.dash_count ) )
     {
         os << " (dash)";
     }
@@ -691,7 +674,7 @@ TextPrinter::printPlayer( std::ostream & os,
     {
         os << " (catch)";
     }
-    else if ( count.move_ != nstohi( player.move_count ) )
+    else if ( count.move_ != rcsc::rcg::nstohi( player.move_count ) )
     {
         os << " (move)";
     }
@@ -702,15 +685,15 @@ TextPrinter::printPlayer( std::ostream & os,
 
     ///////////////////////////////////////////////////////
 
-    if ( count.say_ != nstohi( player.say_count ) )
+    if ( count.say_ != rcsc::rcg::nstohi( player.say_count ) )
     {
         os << " (say)";
     }
-    if ( count.turn_neck_ != nstohi( player.turn_neck_count ) )
+    if ( count.turn_neck_ != rcsc::rcg::nstohi( player.turn_neck_count ) )
     {
         os << " (turn_neck)";
     }
-    if ( count.change_view_ != nstohi( player.change_view_count ) )
+    if ( count.change_view_ != rcsc::rcg::nstohi( player.change_view_count ) )
     {
         os << " (change_view)";
     }
